﻿local addOn, ns = ...
local L = ns.L
_G[addOn] = ns

ns.AddOnVer = '67'
ns.downloadLink = "https://mods.curse.com/addons/wow/loothelper"

local versionStr, internalVersion, dateofpatch, uiVersion = GetBuildInfo(); internalVersion = tonumber(internalVersion)

ns.wowbuild = internalVersion
ns.uibuild	= tonumber(uiVersion)

if ns.uibuild >= 70000 then
	ns.IsLegion	= true
end


-------------------------------------------
-- Legion HookTexture
-------------------------------------------
do
	local tonumber = tonumber
	function ns.CustomSetTexture(obj, a1, a2, a3, a4)
		if ns.IsLegion then
			if ( a1 and a2 and a3 ) and ( tonumber(a1) and tonumber(a2) and tonumber(a3) ) then
				obj:SetColorTexture(a1, a2, a3, a4)
			else
				obj:SetTexture(a1, a2, a3, a4)
			end
		else
			obj:SetTexture(a1, a2, a3, a4)
		end
	end
end

-------------------------------------------
-- Debug
-------------------------------------------

ns.testMode = false
ns.relicTest = true
ns.debugUI = false
ns.showPrint1 = false -- print
ns.showPrint2 = false -- fprint
ns.showPrint3 = false -- dprint

do
	local unpack = unpack
	local type = type
	function ns.inLinePrint(...)
		local t = { ... }
		for i=1, #t do
			if t[i] and type(t[i]) == 'string' then
				t[i] = t[i]:gsub('\n', ' ')
			end
		end
		
		print(unpack(t))
	end
	
	function ns.getLinkCode(msg)
		return msg:gsub('|', 'l')
	end
end

-------------------------------------------
-- Settings
-------------------------------------------

ns.timeWaitForCheck = 2 -- время проверки отдачи вещи
ns.showLootAnonce = true
ns.shortItemLink = false

ns.groupUpSameItems = true

ns.clienRollDice = false

ns.LootRollIconTheme = {}

ns.LootRollIconTheme[1] = { "Interface\\Buttons\\UI-GroupLoot-Dice-Up", "Interface\\Buttons\\UI-GroupLoot-Dice-Highlight", "Interface\\Buttons\\UI-GroupLoot-Dice-Down" }
ns.LootRollIconTheme[2] = { "Interface\\Buttons\\UI-GroupLoot-Coin-Up", "Interface\\Buttons\\UI-GroupLoot-Coin-Highlight", "Interface\\Buttons\\UI-GroupLoot-Coin-Down" }
ns.LootRollIconTheme[3] = { "Interface\\Buttons\\UI-GroupLoot-DE-Up",   "Interface\\Buttons\\UI-GroupLoot-DE-Highlight",   "Interface\\Buttons\\UI-GroupLoot-DE-Down"   }
ns.LootRollIconTheme[4] = { "Interface\\Buttons\\UI-GroupLoot-DE-Up",   "Interface\\Buttons\\UI-GroupLoot-DE-Highlight",   "Interface\\Buttons\\UI-GroupLoot-DE-Down"   }


ns.LootRollButtons = {}
ns.LootRollButtons[1] = { name = L['NEED'],        tip = 'need',       arg = 'roll', icon = 1 }
ns.LootRollButtons[2] = { name = L['OFFSPEC'],      tip = 'offspec',    arg = 'roll', icon = 1 }
ns.LootRollButtons[3] = { name = L['UPGRADE'],        tip = 'upgrade',    arg = 'roll', icon = 2 }
ns.LootRollButtons[4] = { name = L['TRANSMOG'],     tip = 'transmog',   arg = 'roll', icon = 3 }
--ns.LootRollButtons[5] = { name = 'Dis',          tip = 'disenchant', arg = 'normal', icon = 4 }

ns.rollTypeColors = {
	['need']     = "|cFF00FF00",
	['offspec']  = "|cFF00dcff",
	['upgrade']  = '|cFFdcff00',
	['transmog'] = '|cFFf2a0d3',
	['pass']	 = '|cFFFFFFFF',
}

ns.buttonsTexture = 'Interface\\AddOns\\'..addOn..'\\media\\Minimalist.tga'

ns.default_font = 'Interface\\AddOns\\'..addOn..'\\media\\PT-Sans-Narrow-Bold.ttf' --[[Fonts\ARIALN.TTF]] --"Fonts\\FRIZQT___CYR.TTF" --STANDARD_TEXT_FONT
ns.default_font_size = 12
ns.default_font_outline = 'NONE'

ns.arrorTexture = 'Interface\\petbattles\\battlebar-abilitybadge-strong-small'

ns.ShowPass = true

ns.lootQuality = {
	[0] = false, --. Poor (gray): Broken I.W.I.N. Button
	[1] = false, --. Common (white): Archmage Vargoth's Staff
	[2] = false, --. Uncommon (green): X-52 Rocket Helmet
	[3] = true,  --. Rare / Superior (blue): Onyxia Scale Cloak
	[4] = true,  --. Epic (purple): Talisman of Ephemeral Power
	[5] = true,  --. Legendary (orange): Fragment of Val'anyr
	[6] = false, --. Artifact (golden yellow): The Twin Blades of Azzinoth
	[7] = false, --. Heirloom (light yellow): Bloodied Arcanite Reaper
	[8] = false, --. WoW time item
}

ns.subClassItemIgnore = {
	[GetSpellInfo(158716) or 'NULL'] = true,
}
ns.itemIDIgnore = {
	[124124] = true
}

ns.ITEM_LEVEL_PATTERN =  '^'..ITEM_LEVEL:gsub('%%d', '(%%d+)')

function ns.GlobalItemIgnore(link, class, subclass)
	if subclass and ns.subClassItemIgnore[subclass] then
		return true
	end
	
	if link then
		local itemID = ns.GetItemID(link)
		if itemID and ns.itemIDIgnore[itemID] then
			return true
		end
	end
	
	return false
end
-------------------------------------------
-- core.lua
-------------------------------------------

local events = CreateFrame('Frame')
events:SetScript('OnEvent', function(self, event, ...)
	self[event](self, event, ...)
end)

events:RegisterEvent('CHAT_MSG_ADDON')
events:RegisterEvent('ADDON_LOADED')
events:RegisterEvent('PLAYER_LOGIN')

local channelNameLootRollStart = 'LHRStart1'
if not IsAddonMessagePrefixRegistered(channelNameLootRollStart) then
	RegisterAddonMessagePrefix(channelNameLootRollStart)
end

local channelNameLootRollSend = 'LHRSend1'
if not IsAddonMessagePrefixRegistered(channelNameLootRollSend) then
	RegisterAddonMessagePrefix(channelNameLootRollSend)
end

local channelNameLootRollButtons = 'LHRButtons1'
if not IsAddonMessagePrefixRegistered(channelNameLootRollButtons) then
	RegisterAddonMessagePrefix(channelNameLootRollButtons)
end

local channelNameLootRollCancel = 'LHRCancel1'
if not IsAddonMessagePrefixRegistered(channelNameLootRollCancel) then
	RegisterAddonMessagePrefix(channelNameLootRollCancel)
end

local channelNameLootRollDice = 'LHRDice1'
if not IsAddonMessagePrefixRegistered(channelNameLootRollDice) then
	RegisterAddonMessagePrefix(channelNameLootRollDice)
end

local channelNameLootRollAward = 'LHRAward1'
if not IsAddonMessagePrefixRegistered(channelNameLootRollAward) then
	RegisterAddonMessagePrefix(channelNameLootRollAward)
end

local channelNameLootChangeRoll = 'LHRChange1'
if not IsAddonMessagePrefixRegistered(channelNameLootChangeRoll) then
	RegisterAddonMessagePrefix(channelNameLootChangeRoll)
end

local channelNameLootCheck = 'LHRCheck'
if not IsAddonMessagePrefixRegistered(channelNameLootCheck) then
	RegisterAddonMessagePrefix(channelNameLootCheck)
end

local channelNameLockOut = 'LHRLock'
if not IsAddonMessagePrefixRegistered(channelNameLockOut) then
	RegisterAddonMessagePrefix(channelNameLockOut)
end

local channelNameLockOutR = 'LHRLockR'
if not IsAddonMessagePrefixRegistered(channelNameLockOutR) then
	RegisterAddonMessagePrefix(channelNameLockOutR)
end

ns.channelNameLootRollStart   = channelNameLootRollStart
ns.channelNameLootRollSend    = channelNameLootRollSend
ns.channelNameLootRollButtons = channelNameLootRollButtons
ns.channelNameLootRollCancel  = channelNameLootRollCancel
ns.channelNameLootRollDice    = channelNameLootRollDice
ns.channelNameLootRollAward   = channelNameLootRollAward
ns.channelNameLootChangeRoll  = channelNameLootChangeRoll
ns.channelNameLootCheck  	  = channelNameLootCheck
ns.channelNameLockOut		  = channelNameLockOut
ns.channelNameLockOutR		  = channelNameLockOutR

function ns.print(...)
	if not ns.testMode then return end
	if not ns.showPrint1 then return end
	if ns.debugUI then
		ns.Debug_AddMessage(...)
	else
		print(...)
	end
end

function ns.fprint(...)
	if not ns.testMode then return end
	if not ns.showPrint2 then return end
	if ns.debugUI then
		ns.Debug_AddMessage(...)
	else
		print(...)
	end
end

function ns.dprint(...)
	if not ns.showPrint3 then return end
	if ns.debugUI then
		ns.Debug_AddMessage(...)
	else
		print(...)
	end
end

function ns.ConfirmReceive()
	ns:SendMessage('CONFIRM', channelNameLootCheck)
end

function ns.ResendRequest()
	ns:SendMessage('RESEND', channelNameLootCheck)
end

local msgSendCache = {}
local msgPartTag = 'SH:%d:%d:%s'
local maxMsgLen = 225

function ns:SendMessage(msg, tag)
	local channel = nil

	if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE) then
		channel = "INSTANCE_CHAT"
	elseif IsInRaid(LE_PARTY_CATEGORY_HOME) then
		channel = "RAID"
	elseif IsInGroup(LE_PARTY_CATEGORY_HOME) then
		channel = "PARTY"
	end

	if not channel then
		if ns.testMode then 
			channel = 'GUILD'
		else
			return 
		end
	end
	
	local msgLen = msg:len()	
	local numParts = ceil(msgLen/maxMsgLen)
	
--	print('T2', ns.inLinePrint(msg), msgLen)
	
	for i=1, numParts do
		local msg = format(msgPartTag, i, numParts, msg:sub( (maxMsgLen*i)-maxMsgLen+1, ( i == numParts and msgLen or maxMsgLen*i)))
		
		SendAddonMessage(tag, msg, channel)
	end
end

function ns.SendMessageGuild(msg, tag)	
	local msgLen = msg:len()	
	local numParts = ceil(msgLen/maxMsgLen)

	for i=1, numParts do
		local msg = format(msgPartTag, i, numParts, msg:sub( (maxMsgLen*i)-maxMsgLen+1, ( i == numParts and msgLen or maxMsgLen*i)))
		
		SendAddonMessage(tag, msg, 'GUILD')
	end
end

local buildInMsg = {}
local function IsResetBeforeDone(author)
	local i = 0
	for _, _ in pairs(buildInMsg[author]) do
		i = i+1
	end
	
	return i
end

function ns:RecieveMessage(msg, author, tag)
--	if not buildInMsg[author] then
--		print('author', author)
--	end
	buildInMsg[author] = buildInMsg[author] or {}

	local part, totalParts, datas = string.match(msg, 'SH:(%d+):(%d+):(.+)')

	part = tonumber(part)
	totalParts = tonumber(totalParts)

	if part == 1 then	
	--	local num = IsResetBeforeDone(author)
	--	if num > 0 then
	--		print('Error on recieve msg', num)
	--	end
		
		wipe(buildInMsg[author])
	end

	buildInMsg[author][part] = datas

	if part == totalParts then
	--	local num = IsResetBeforeDone(author)		
		
--		print('T', author, 'curParts:', num, 'part:',part, 'totalParts:',totalParts)
		
		local cmsg = ''
		
		for i=1, #buildInMsg[author] do
			cmsg = cmsg..buildInMsg[author][i]
		end
		wipe(buildInMsg[author])

		if tag == channelNameLootRollStart then
			ns:ShowLootItems(cmsg, author)
			ns:ShowLootRollsItemsList(cmsg, author)
			
			C_Timer.After(1, ns.ConfirmReceive)
		elseif tag == channelNameLootRollSend then
			ns:ShowLootRolls(cmsg, author)
			if ns.testMode then
				for i=1, 5 do
					ns:ShowLootRolls(cmsg, i..author)
				end
				
				C_Timer.After(1, function()
					for i=6, 10 do
						ns:ShowLootRolls(cmsg, i..author)
					end
				end)
				
				C_Timer.After(2, function()
					for i=11, 16 do
						ns:ShowLootRolls(cmsg, i..author)
					end
				end)
				
				C_Timer.After(3, function()
					for i=17, 21 do
						ns:ShowLootRolls(cmsg, i..author)
					end
				end)
			end
		elseif tag == channelNameLootRollButtons then
			ns:SetButtonSettings(cmsg, author)
		elseif tag == channelNameLootRollCancel then
			ns:HideLootItems(cmsg, author)
			ns:HideLootRollsItemsList(cmsg, author)
		elseif tag == channelNameLootRollDice then	
			ns:UpdateRollDice(cmsg, author)
		elseif tag == channelNameLootRollAward then
			ns.History_ReceiveData(cmsg, author)
		elseif tag == channelNameLootChangeRoll then
			ns.UpdateRollType_OnChange(cmsg, author)
		elseif tag == channelNameLootCheck then
			-- RL Status
			ns:Server_ReceiveHandler(cmsg, author)
		elseif tag == channelNameLockOut then
			ns:Client_ReceiveEncounterLockout(cmsg, author, tag)
		elseif tag == channelNameLockOutR then
			ns:Client_SendEncounterLockout(cmsg, author, tag)
		elseif tag == ns.channelNameRelicNewInfo then	
			ns.RelicSync_GetNewItemInfo(cmsg, author, tag)
		end
	end
end

function events:CHAT_MSG_ADDON(event, tag, msg, channel, author, authorShort)

	if tag == ns.channelNameRelicNewInfo then
		ns:RecieveMessage(msg, author, tag)	
	end
	
	if not ns.testMode and channel == 'GUILD' then return end
	if not ns.db.enable then return end

	if tag == channelNameLootRollStart then
		ns:EndTest()
		
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootRollSend then
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootRollButtons then
		ns:EndTest()
		
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootRollCancel then	
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootRollDice then
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootRollAward then
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootChangeRoll then
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLootCheck then
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLockOut then
		ns:RecieveMessage(msg, author, tag)
	elseif tag == channelNameLockOutR then
		ns:RecieveMessage(msg, author, tag)		
	end
end
local default_settings = {
	ServerSizeMover = { 0, 0 },
	LootRollMover   = { 0, 0 },
	ClientSizeMover = { 0, 0 },
	LootListMover = { 0, 0 },
	
	enable = true,
	enableServer = true,
	showPass = true,
	ignoreUnusableItems = false,
	ShownMainFrame = true,
	RightClickToHideAllRolls = false,
	RollsOnTop = true,
	ShowItemLevel = true,
	SearchSameSession = true,
	EnableLootRollLog = true,
	AutoClearLootLog = true,
	ColoredRollFilter = true,
	AuraPassUnusableRolls = false,
	EnableTokenItemHook = false,
	EnableAwardingLoot = false,
	EnableShornLootLink = false,
	relicTooltipInfoHelp = true,
	groupUpSameItems = true,
	
	enableScreenShot = false,
	
	doStartRollAnonce = true,
	
	enableAlerts = true,
	
	ignorePerZoneClient = {	
		{	
			name = 'HFC',
			zoneID = 1448,
			values = {
				{ name = 'LFR',	   diff = 17, value = false, },
				{ name = 'NORMAL', diff = 14, value = true, },
				{ name = 'HEROIC', diff = 15, value = true, },
				{ name = 'MYTHIC', diff = 16, value = true, },
			},		
		},
	},
	
	ignorePerZoneServer = {		
		{	
			name = 'HFC',
			zoneID = 1448,
			values = {
				{ name = 'LFR',	   diff = 17, value = false, },
				{ name = 'NORMAL', diff = 14, value = true, },
				{ name = 'HEROIC', diff = 15, value = true, },
				{ name = 'MYTHIC', diff = 16, value = true, },
			},		
		},
	},
	
	LootRollButtonsNumButtons = 4,
	LootRollButtons = {
		{ name = L['NEED'],        tip = 'need',       arg = 'roll', icon = 1 },
		{ name = L['OFFSPEC'],      tip = 'offspec',    arg = 'roll', icon = 1 },
		{ name = L['UPGRADE'],        tip = 'upgrade',    arg = 'roll', icon = 2 },
		{ name = L['TRANSMOG'],     tip = 'transmog',   arg = 'roll', icon = 3 },
		{ name = L['NEED'],        tip = 'need',       arg = 'roll', icon = 1 },
		{ name = L['NEED'],        tip = 'need',       arg = 'roll', icon = 1 },		
	},
	
	lootQuality = {
		[0] = false, --. Poor (gray): Broken I.W.I.N. Button
		[1] = false, --. Common (white): Archmage Vargoth's Staff
		[2] = false, --. Uncommon (green): X-52 Rocket Helmet
		[3] = true,  --. Rare / Superior (blue): Onyxia Scale Cloak
		[4] = true,  --. Epic (purple): Talisman of Ephemeral Power
		[5] = true,  --. Legendary (orange): Fragment of Val'anyr
		[6] = false, --. Artifact (golden yellow): The Twin Blades of Azzinoth
		[7] = false, --. Heirloom (light yellow): Bloodied Arcanite Reaper
	--	[8] = false, --. WoW time item
	},
	
	rollTypeColors = {
		['need']     = "|cFF00FF00",
		['offspec']  = "|cFF00dcff",
		['upgrade']  = '|cFFdcff00',
		['transmog'] = '|cFFf2a0d3',
		['pass']	 = '|cFFFFFFFF',
	},
	
	lootHistory_Date = {},
	lootHistory_Units = {},
	lootHistory_Date_Units = {},
	lootHistory_Encounter_Boss = {},
	lootHistory = {},
	
	lootFilterCache = {},
}

ns.default_settings = default_settings

local minimapButton = nil

local function GetMinimapButtonMenuList()
	local list = {}
	
	list[#list+1] = { text = NORMAL_FONT_COLOR_CODE..addOn..' v'..ns.AddOnVer..FONT_COLOR_CODE_CLOSE,  disabled = true, isTitle = true, notCheckable = true}
	list[#list+1] =	{ text = L['OPEN_SESSION'], func = function(self) ns.OpenLootRollsMainFrame()  end, notCheckable = true }
	list[#list+1] =	{ text = '', disabled = true, notCheckable = true  }
	
	if ns:IsMasterLooter() then 
		list[#list+1] =	{ text = ( ns.db.enableScreenShot and L['DISABLE_SCREENSHOTS'] or L['ENABLE_SCREENSHOTS']), func = function(self) ns.db.enableScreenShot = not ns.db.enableScreenShot end, notCheckable = true }
		list[#list+1] =	{ text = '', disabled = true, notCheckable = true }
	end
	
	list[#list+1] =	{ text = L['CANCEL'], func = function(self) ns.HideMenu() end, notCheckable = true }

	
	return list
end

local function ShowHideUI(command, ...)
	
	if command and command == 'RightButton' then
		ns.ShowMenu(minimapButton, GetMinimapButtonMenuList()) --:SetParent(minimapButton)
		return
	end
	
	if command and command:sub(1, 7) == 'session' then
		ns.OpenLootRollsMainFrame()
		ns.HideMenu()
		return
	elseif command and command:sub(1, 6) == 'resend' then
		ns.ResendRequest()
		return
	end
	
	ns.HideMenu()
	
	if AleaUI_GUI:IsOpened(addOn) then
		AleaUI_GUI:Close(addOn)
	else
		AleaUI_GUI:Open(addOn)
	end
end

local function OnClick_MinimapButton(self, button)
	if self.OnClick then	
		self.OnClick(button)
	end
end

function events:ADDON_LOADED(event, loaded_addOn)
	if addOn ~= loaded_addOn then return end
	
	ns.db = ALEAUI_NewDB("LootHelperDB", default_settings, true)
	
	ns.db.minimap = ns.db.minimap or {}
	
	AleaUI_GUI.SlashCommand(addOn, "/loothelper", ShowHideUI)
	AleaUI_GUI.MinimapButton(addOn, { OnClick = ShowHideUI, texture = "Interface\\Buttons\\UI-GroupLoot-Dice-Up" }, ns.db.minimap )
	
	minimapButton = AleaUI_GUI.GetMinimapButton(addOn)
	minimapButton:SetScript('OnClick', OnClick_MinimapButton)

	if LootHelperDB.ClientSizeMover then	
		ns.db.ClientSizeMover = LootHelperDB.ClientSizeMover
		LootHelperDB.ClientSizeMover = nil
	end
	
	if LootHelperDB.ServerSizeMover then	
		ns.db.ServerSizeMover = LootHelperDB.ServerSizeMover
		LootHelperDB.ServerSizeMover = nil
	end
	
	if LootHelperDB.LootRollMover then	
		ns.db.LootRollMover = LootHelperDB.LootRollMover
		LootHelperDB.LootRollMover = nil
	end
	
	ns:UpdateServerSizePosition()
	ns:UpdateClientSizePosition()
	ns:UpdateLootRollsSizePosition()
	ns:UpdateLootListSizePosition()
	
	ns:UpdateButtonListTable()
end

function events:PLAYER_LOGIN()
	if ns.db.AutoClearLootLog then
		ns.History_AutoClear()
	end
	
	ns.BuildClassItemFilter()
	ns.Cache_InitUpdateLocale()
	
--	GameTooltip:HookScript("OnTooltipSetItem", ns.GameTooltip_OnTooltipSetItem)
	GameTooltip:HookScript("OnShow", ns.GameTooltip_OnTooltipSetItem)
	GameTooltip:HookScript("OnHide", ns.HideRelicInfoFrame)
end

do
	local unpack = unpack
	local GetItemInfo = GetItemInfo
	local tremove = table.remove
	
	local queueforItems = {}
	local handler = CreateFrame("Frame")
	handler:RegisterEvent('GET_ITEM_INFO_RECEIVED')
	
	local function CheckForItemGained(self, event, itemID)
	
		local index = 1
	
		while queueforItems[index] do
			local d = queueforItems[index]
			local checkItem = GetItemInfo(d.link)
			if checkItem then
				ns.print('RecieveItems', d.link)
				ns[d.func](d.link, unpack(d.args))
				tremove(queueforItems, index)
			else
				index = index + 1
			end
		end
		
		--[==[
		for i=1, #queueforItems do
			local d = queueforItems[i]
			local checkItem = GetItemInfo(d.link)		
			if checkItem then					
				ns.print('RecieveItems', d.link)
				ns[d.func](d.link, unpack(d.args))
				tremove(queueforItems, i)
				CheckForItemGained()
				break
			end
		end
		]==]
	end	
	handler:SetScript('OnEvent', CheckForItemGained)

	function ns.QueueForItemInfo(link, func, ...)
		local checkItem = GetItemInfo(link)
		
		if not checkItem then
			queueforItems[#queueforItems+1] = { 
				link = link, 
				func = func, 
				args = { ... } 
			}
		else
			ns[func](link, ...)
		end
	end
	
--	QueueForItemInfo = ns.QueueForItemInfo
end

do
	local unpack = unpack
	local tremove = table.remove
	local GetTime = GetTime
	
	local amountIndex = 0
	
	local queueforDelays = {}
	local handler = CreateFrame("Frame")
		handler:Hide()
		
	local function CheckForQueue(timer)	
		
		local index = 1
		
		while queueforDelays[index] do
			local d = queueforDelays[index]

			if d.endTime < timer then
				if type(d.func) == 'function' then
					d.func(unpack(d.args))
				elseif ns[d.func] then
					ns[d.func](unpack(d.args))
				end
			
				tremove(queueforDelays, index)
			else
				index = index + 1
			end
		end
		--[==[
		for i=1, #queueforDelays do
			local d = queueforDelays[i]	
			
			if d.endTime < timer then
			
				if type(d.func) == 'function' then
					d.func(unpack(d.args))
				elseif ns[d.func] then
					ns[d.func](unpack(d.args))
				end
			
				tremove(queueforDelays, i)
				CheckForQueue(timer)			
				break
			end
		end
		]==]
	end	
	
	local function CheckForQueueByName(name)	
	
		local index = 1
		
		while queueforDelays[index] do
			local d = queueforDelays[index]	
			
			if d.func == name then
				tremove(queueforDelays, index)
			else
				index = index + 1;
			end	
		end
		
		--[==[
		for i=1, #queueforDelays do
			local d = queueforDelays[i]	
			
			if d.func == name then		
				tremove(queueforDelays, i)
				CheckForQueueByName(name)			
				break
			end
		end	
		]==]
	end
	
	handler:SetScript('OnUpdate', function(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed
		
		if self.elapsed < 0.1 then return end
		self.elapsed = 0
		
		if #queueforDelays == 0 then self:Hide() end
	
		CheckForQueue(GetTime())
	end)
	
	function ns.QueueFunction(timer, func, ...)
		local index = #queueforDelays+1
		amountIndex = amountIndex + 1
		
		queueforDelays[index] = { 
			endTime = ( GetTime() + timer ),
			amountIndex = amountIndex,
			func = func, 
			args = { ... } 
		}
		
		handler.elapsed = 0
		handler:Show()
		
		return amountIndex
	end
	
	function ns.UpdateQueueFunction(amountIndex, timer, func, ...)
		local success = false
		for i=1, #queueforDelays do			
			if queueforDelays[i].amountIndex == amountIndex then
				queueforDelays[i].endTime = ( GetTime() + timer )
				queueforDelays[i].args = { ... } 	
				queueforDelays[i].func = func or queueforDelays[i].func
				
				success = true
				
				break
			end
		end
		
		handler.elapsed = 0
		handler:Show()

		return success, amountIndex
	end
	
	function ns.BreakQueueFunctionByID(amountIndex)
		for i=1, #queueforDelays do			
			if queueforDelays[i].amountIndex == amountIndex then
				tremove(queueforDelays, i)
				break
			end
		end
	end
	
	function ns.BreakQueueFunctionByName(name)
		CheckForQueueByName(name)
	end
	
	function ns.GetQueueFunctionArgs(amountIndex)
		for i=1, #queueforDelays do			
			if queueforDelays[i].amountIndex == amountIndex then
				return unpack(queueforDelays[i].args)
			end
		end
	end
end


do
	local GetInstanceInfo = GetInstanceInfo

	
	function ns:IsInstanceIgnored_Client()
		local name, type, difficulty, _, _, _, _, mapID = GetInstanceInfo()
	
		for i=1, #ns.db.ignorePerZoneClient do
			local data = ns.db.ignorePerZoneClient[i]			
			if data.zoneID == mapID then		
				if mapID == 1448 then
					local isShown = false
					for a=1, #data.values do						
						if difficulty == data.values[a].diff then
							return data.values[a].value
						end
					end
				end
			end
		end
		
		return true
	end
	
	function ns:IsInstanceIgnored_Server()
		local name, type, difficulty, _, _, _, _, mapID = GetInstanceInfo()
	
		for i=1, #ns.db.ignorePerZoneServer do
			local data = ns.db.ignorePerZoneServer[i]			
			if data.zoneID == mapID then		
				if mapID == 1448 then
					local isShown = false
					for a=1, #data.values do						
						if difficulty == data.values[a].diff then
							return data.values[a].value
						end
					end
				end
			end
		end
		
		return true
	end
end

do
	local session = 'LootTesting'
	local links = {
		"\124cffa335ee\124Hitem:87777:0:0:0:0:0:0:0:0:0:0\124h[Reins of the Astral Cloud Serpent]\124h\124r",
	--	"\124cffa335ee\124Hitem:124519:0:0:0:0:0:0:0:0:0:529:0:1:0\124h[Repudiation of War]\124h\124r",
		"\124cffa335ee\124Hitem:127959:0:0:0:0:0:0:0:0:0:0:0:1:0\124h[Helm of Hellfire's Vanquisher]\124h\124r",
		"\124cffa335ee\124Hitem:127953:0:0:0:0:0:0:0:0:0:0:0:1:0\124h[Нагрудник завоевателя Адского Пламени]\124h\124r",
	--	"\124cffa335ee\124Hitem:116660:0:0:0:0:0:0:0:0:0:0\124h[Ironhoof Destroyer]\124h\124r",
		"\124cffa335ee\124Hitem:124367:0:0:0:0:0:0:0:0:0:529:0:2:562:567\124h[Fang of the Pit]\124h\124r",
		"\124cffa335ee\124Hitem:124361:0:0:0:0:0:0:0:0:0:529:0:1:567\124h[Cursed Demonbone Longbow]\124h\124r",
	--	"\124cffa335ee\124Hitem:124382:0:0:0:0:0:0:0:0:0:529:0:1:567\124h[Edict of Argus]\124h\124r",
		"\124cffa335ee\124Hitem:124159:0:0:0:0:0:0:0:0:0:529:0:1:567\124h[Demon Prince's Ascendant Crown]\124h\124r",
	--	"\124cffa335ee\124Hitem:124330:0:0:0:0:0:0:0:0:0:529:0:1:567\124h[Helm of Precognition]\124h\124r",
		"\124cffa335ee\124Hitem:124355:0:0:0:0:0:0:0:0:0:529:0:1:567\124h[Fallen Defender of Argus]\124h\124r",
		"\124cffa335ee\124Hitem:113882:0:0:0:0:0:0:0:0:0:0:0:1:0\124h[Glutton's Kerchief]\124h\124r",
	}
	
	local info = session..'\n'..links[1]..'\n'..links[2]..'\n'..links[3]..'\n'..links[4]..'\n'..links[5]..'\n'..links[6]..'\n'..links[7]--..'\n'..links[8]..'\n'..links[9]
	
	local author = 'TESTER_MODE'
	local test = false
	
	function ns:StartTest()
		if not test then
			test = true
			
			ns:SendTestRollButtons()
			ns:ShowLootItems(info, author)
			ns:ShowLootRollsItemsList(info, author)
		end
	end
	
	function ns:EndTest()	
		if test then
			test = false
			
			ns:HideLootItems(session, author)
			ns:HideLootRollsItemsList(session, author)
			ns:ClearAllIgnoredRollIDs(session)
		end
	end
	
	function ns:ToggleTest()		
		if not test then
			ns:StartTest()
		else
			ns:EndTest()	
		end
	end
	
	function ns:SendTestRollButtons()
		local msg = ''
		
		for i=1, #ns.LootRollButtons do
			local data = ns.LootRollButtons[i]
		
			msg = format("%s%s\n%s\n%s\n%d\n", msg, data.name, data.tip, data.arg, data.icon)	
		end

		ns:SetButtonSettings(msg, author)
	end

end

do

	local char
	local string_byte = string.byte
	local sub = sub
	
	local function chsize(char)
		if not char then
			return 0
		elseif char > 240 then
			return 4
		elseif char > 225 then
			return 3
		elseif char > 192 then
			return 2
		else
			return 1
		end
	end

	function ns:utf8sub(str, startChar, numChars)
	  if not str then return "" end
	  local startIndex = 1
	  while startChar > 1 do
		  local char = string_byte(str, startIndex)
		  startIndex = startIndex + chsize(char)
		  startChar = startChar - 1
	  end
	 
	  local currentIndex = startIndex
	 
	  while numChars > 0 and currentIndex <= #str do
		local char = string_byte(str, currentIndex)
		currentIndex = currentIndex + chsize(char)
		numChars = numChars -1
	  end
	  return str:sub(startIndex, currentIndex - 1)
	end
end

do
	local unknownColor2 = "FFA6A6A6"
	local UnitClass = UnitClass
	local Ambiguate = Ambiguate
	local UnitName = UnitName
	local RAID_CLASS_COLORS = RAID_CLASS_COLORS
	local MAX_CLASSES = MAX_CLASSES
	
	local classTag_to_ID = {}
	
	for i=1, MAX_CLASSES do		
		local _, class = GetClassInfoByID(i);
		if class then
			classTag_to_ID[class] = i
		end
	end
	
	function ns.GetClassInfo( class )
		local classID = classTag_to_ID[class]
	
		classID = classID or class
		
		local className, classFile, index = GetClassInfoByID(classID);
		
		return className, classFile, index, RAID_CLASS_COLORS[classFile].colorStr
	end
	
	function ns.GetColoredNameByClassFile(unit, classFile)
		
		local tUnit = Ambiguate(unit, "none")
		
		local short = UnitName(tUnit)
		if not short then
			short = Ambiguate(unit, "short")
		end
		
		local color = unknownColor2
		if classFile and RAID_CLASS_COLORS[classFile]then
			color = RAID_CLASS_COLORS[classFile].colorStr or unknownColor2
		end	
	
		return '|c'..color..short..'|r'
		
	end
	
	function ns.GetColoredName(unit)
	
		local tUnit = Ambiguate(unit, "none")
		local _, class = UnitClass(tUnit)
		
		local short = UnitName(tUnit)
		if not short then
			short = Ambiguate(unit, "short")
		end
		
		local color = unknownColor2
		if class then
			color = RAID_CLASS_COLORS[class] and RAID_CLASS_COLORS[class].colorStr or unknownColor2
		end	
	
		return '|c'..color..short..'|r'
	end
	
	function ns.GetColoredNameByID(unit, classID)
		
	--	print('T', 'GetColoredNameByID', unit, classID)
		local tUnit = Ambiguate(unit, "none")
		
		local _, class = GetClassInfoByID(classID);
		
		local short = UnitName(tUnit)
		if not short then
			short = Ambiguate(unit, "short")
		end
		
		local color = unknownColor2
		if class then
			color = RAID_CLASS_COLORS[class] and RAID_CLASS_COLORS[class].colorStr or unknownColor2
		end	
	
		return '|c'..color..short..'|r'
	
	end
end

do
	-- /script DEFAULT_CHAT_FRAME:AddMessage("\124cffa335ee\124Hitem:124160:0:0:0:0:0:0:0:0:0:529:0:1:0\124h[Cowl of the Arcanic Conclave]\124h\124r");
	-- /script DEFAULT_CHAT_FRAME:AddMessage("\124cffa335ee\124Hitem:124160:0:0:0:0:0:0:0:0:0:529:0:1:566\124h[Cowl of the Arcanic Conclave]\124h\124r");
	local linkPattern = ns.IsLegion and '|Hitem:%d::::::::::529::1:%d|h[L]|h' or '|Hitem:%d:0:0:0:0:0:0:0:0:0:529:0:1:%d|h[L]|h'
	
	local tierTokenItemList = {
		-- HFC
		-- Head
		[127959] = {
			['MAGE'] = 124160,
			['ROGUE'] = 124263,
			['DEATHKNIGH'] = 124332,
			['DRUID'] = 124261,		
		}, -- Rogue, Death Knight, Mage, Druid 
		[127966] = {
			['WARRIOR'] = 124334,
			['HUNTER'] = 124296,
			['SHAMAN'] = 124297,
			['MONK'] = 124262,
		}, -- Warrior, Hunter, Shaman, Monk
		[127956] = {
			['PALADIN'] = 124333,
			['PRIEST'] = 124161,
			['WARLOCK'] = 124162,
		}, -- Paladin, Priest, Warlock
		
		-- Legs
		[127955] = {
			['PALADIN'] = 124339,
			['PRIEST'] = 124166,
			['WARLOCK'] = 124167,
		}, -- Paladin, Priest, Warlock
		[127960] = {
			['MAGE'] = 124165,
			['ROGUE'] = 124269,
			['DEATHKNIGH'] = 124338,
			['DRUID'] = 124267,		
		}, -- Rogue, Death Knight, Mage, Druid
		[127965] = {
			['WARRIOR'] = 124340,
			['HUNTER'] = 124301,
			['SHAMAN'] = 124302,
			['MONK'] = 124268,
		}, -- Warrior, Hunter, Shaman, Monk
		
		-- Hands
		[127954] = {
			['PALADIN'] = 124328,
			['PRIEST'] = 124155 ,
			['WARLOCK'] = 124156,
		}, -- Paladin, Priest, Warlock
		[127958] = {
			['MAGE'] = 124154,
			['ROGUE'] = 124257,
			['DEATHKNIGH'] = 124327,
			['DRUID'] = 124255,		
		}, -- Rogue, Death Knight, Mage, Druid
		[127964] = {
			['WARRIOR'] = 124329,
			['HUNTER'] = 124292,
			['SHAMAN'] = 124293,
			['MONK'] = 124256,
		}, -- Warrior, Hunter, Shaman, Monk
		
		-- Shoulders
		[127967] = {
			['WARRIOR'] = 124346,
			['HUNTER'] = 124307,
			['SHAMAN'] = 124308,
			['MONK'] = 124273,
		},	-- Warrior, Hunter, Shaman, Monk
		[127961] = {
			['MAGE'] = 124177,
			['ROGUE'] = 124274,
			['DEATHKNIGH'] = 124344,
			['DRUID'] = 124272,		
		}, -- Rogue, Death Knight, Mage, Druid
		[127957] = {
			['PALADIN'] = 124345,
			['PRIEST'] = 124178,
			['WARLOCK'] = 124179,
		}, -- Paladin, Priest, Warlock
		
		-- Chest
		[127962] = {
			['MAGE'] = 124171,
			['ROGUE'] = 124248,
			['DEATHKNIGH'] = 124317,
			['DRUID'] = 124246,		
		}, -- Rogue, Death Knight, Mage, Druid
		[127963] = {
			['WARRIOR'] = 124319,
			['HUNTER'] = 124284,
			['SHAMAN'] = 124303,
			['MONK'] = 124247,
		}, -- Warrior, Hunter, Shaman, Monk
		[127953] = {
			['PALADIN'] = 124318,
			['PRIEST'] = 124172,
			['WARLOCK'] = 124173,
		}, -- Paladin, Priest, Warlock

		-- Trinket
		[127970] = {
			['WARRIOR'] = 124523,
			['HUNTER'] = 124515,
			['SHAMAN'] = 124521,
			['MONK'] = 124517,
		}, -- Warrior, Hunter, Shaman, Monk
		[127968] = {
			['MAGE'] = 124516,
			['ROGUE'] = 124520,
			['DEATHKNIGH'] = 124513,
			['DRUID'] = 124514,		
		}, -- Rogue, Death Knight, Mage, Druid
		[127969] = {
			['PALADIN'] = 124518,
			['PRIEST'] = 124519,
			['WARLOCK'] = 124522,
		}, -- Paladin, Priest, Warlock
		--[==[
		-- Nighthold
		-- Head
		[143565] = 'INVTYPE_HEAD',
		[143575] = 'INVTYPE_HEAD',
		[143568] = 'INVTYPE_HEAD',
		
		-- Legs
		[143564] = 'INVTYPE_LEGS',
		[143574] = 'INVTYPE_LEGS',
		[143569] = 'INVTYPE_LEGS',
		
		-- Hands
		[143563] = 'INVTYPE_HAND',
		[143573] = 'INVTYPE_HAND',
		[143567] = 'INVTYPE_HAND',
		
		-- Shoulders
		[143566] = 'INVTYPE_SHOULDER',
		[143576] = 'INVTYPE_SHOULDER',
		[143570] = 'INVTYPE_SHOULDER',
		
		-- Chest
		[143562] = 'INVTYPE_CHEST',
		[143572] = 'INVTYPE_CHEST',
		[143571] = 'INVTYPE_CHEST',
		
		-- Back
		[143577] = 'INVTYPE_CLOAK',
		[143579] = 'INVTYPE_CLOAK',
		[143578] = 'INVTYPE_CLOAK',
		
		-- TOS 
		-- Head
		[147323] = 'INVTYPE_HEAD',
		[147324] = 'INVTYPE_HEAD',
		[147322] = 'INVTYPE_HEAD',
		
		-- Legs
		[147327] = 'INVTYPE_LEGS',
		[147326] = 'INVTYPE_LEGS',
		[147325] = 'INVTYPE_LEGS',
		
		-- Hands
		[147321] = 'INVTYPE_HAND',
		[147320] = 'INVTYPE_HAND',
		[147319] = 'INVTYPE_HAND',
		
		-- Shoulders
		[147330] = 'INVTYPE_SHOULDER',
		[147329] = 'INVTYPE_SHOULDER',
		[147328] = 'INVTYPE_SHOULDER',
		
		-- Chest
		[147318] = 'INVTYPE_CHEST',
		[147317] = 'INVTYPE_CHEST',
		[147316] = 'INVTYPE_CHEST',
		
		-- Back
		[147333] = 'INVTYPE_CLOAK',
		[147332] = 'INVTYPE_CLOAK',
		[147331] = 'INVTYPE_CLOAK',
		]==]
	}
	
	local MYTHIC_KEYWORD = PLAYER_DIFFICULTY6
	local HEROIC_KEYWORD = PLAYER_DIFFICULTY2
	
	local function IsMythicToken(link)		
		local checkForMythic = ns.CheckForContainTooltip(link, MYTHIC_KEYWORD)		
		return checkForMythic or false
	end
	
	local function IsHeroicToken(link)		
		local checkForHeroic = ns.CheckForContainTooltip(link, HEROIC_KEYWORD)		
		return checkForHeroic or false
	end
	
	local function HideTip(frame) GameTooltip:Hide(); frame:UnregisterEvent('MODIFIER_STATE_CHANGED'); ns.HideRelicInfoFrame() end
	local function HideTip2(frame) GameTooltip:Hide(); ResetCursor(); frame:UnregisterEvent('MODIFIER_STATE_CHANGED'); ns.HideRelicInfoFrame() end
	
	local showedRelics = {}
		
	local function AddRelic(link, itemID, index)	
		showedRelics[index] = true
		
		local quality, ilvl,_, _, _, _, _, texture = select(3, GetItemInfo(link))
		local relicID = ns.GetItemID(link)
		local spellID = ns.SpellIDByRelic[itemID..'.'..relicID]
		local spellName, desc
		
		if spellID and GetSpellInfo(spellID) then
			spellName = GetSpellInfo(spellID)
			desc = GetSpellDescription(spellID)
		else
			spellName = 'Spell name is nil for >'..relicID..'-'..itemID..'<'
			desc = 'Desc is nil for spell:'..(spellID or 'nil')
		end
			
		local relicType = (select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(link))))
		local r, g, b, hex = GetItemQualityColor(quality)
		
		GameTooltip:AddDoubleLine('\124T'..texture..':22:22:0:0:64:64:4:60:4:60\124t'..'|c'..hex..'('..ilvl..')|r'..spellName, ns.RelicTypes[relicType], 1,1,1, 0, 1, 0)
		GameTooltip:AddLine(' '..desc, 1,1,1, true)
		GameTooltip:AddLine(' ')
		
		GameTooltip:Show()
	end
	
	local function ShowArtifactTooltip(itemID, link)
		local name = GetItemInfo(itemID)
		local iLevel = 750
		
		local _, relicLink1 = GetItemGem(link, 1)
		local _, relicLink2 = GetItemGem(link, 2)
		local _, relicLink3 = GetItemGem(link, 3)
		
		local relicLink1_retry = true
		local relicLink2_retry = true
		local relicLink3_retry = true
		
		local r, g, b, hex = GetItemQualityColor(6)
		
		local itemLevel = ns.CheckForContainTooltip(link, ns.ITEM_LEVEL_PATTERN) or ''
		
		GameTooltip:AddLine('['..name..'] '..itemLevel, r, g, b)
		
		local id, specName, description, icon, role, class = GetSpecializationInfoByID(ns.Artifacts[itemID])
		
		local classDisplayName, classTag, classID
		local numClasses = GetNumClasses();
		for i = 1, numClasses do
			classDisplayName, classTag, classID = GetClassInfo(i);
			if classTag == class then
				break
			end
		end
		
		local color = RAID_CLASS_COLORS[classTag]
		
		GameTooltip:AddLine('|T'..icon..':0|t'..specName, color.r, color.g, color.b)
		GameTooltip:AddLine(' ')

	--	print('T1', link, relicLink1,relicLink2,relicLink3) 
	--	print(GetSpecializationInfoByID(ns.Artifacts[itemID]))
	--	print(specName, classDisplayName, icon, role, class)
		
	--	GameTooltip:AddLine(' ')
		
		if ns.SpellIDByRelic then
			wipe(showedRelics)
			
			if relicLink1 then
				AddRelic(relicLink1, itemID, 1)
			else
				C_Timer.After(0.2, function()
					if not showedRelics[1] then
						AddRelic((select(2, GetItemGem(link, 1))), itemID, 1)
					end
				end)
			end
			
			if relicLink2 then
				AddRelic(relicLink2, itemID, 2)
			else
				C_Timer.After(0.2, function()
					if not showedRelics[2] then
						AddRelic((select(2, GetItemGem(link, 2))), itemID, 2)
					end
				end)
			end
			
			if relicLink3 then
				AddRelic(relicLink3, itemID, 3)
			else
				C_Timer.After(0.2, function()
					if not showedRelics[3] then
						AddRelic((select(2, GetItemGem(link, 3))), itemID, 3)
					end
				end)
			end
			
			--[==[
			if relicLink2 then
				local quality, ilvl,_, _, _, _, _, texture = select(3, GetItemInfo(relicLink2))
				local spellID = ns.SpellIDByRelic[itemID..'.'..ns.GetItemID(relicLink2)]
				local spellName = GetSpellInfo(spellID)
				local desc = GetSpellDescription(spellID)
				local relicType = (select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink2))))
				local r, g, b, hex = GetItemQualityColor(quality)
				
				GameTooltip:AddDoubleLine('\124T'..texture..':22:22:0:0:64:64:4:60:4:60\124t'..'|c'..hex..'('..ilvl..')|r'..spellName, ns.RelicTypes[relicType], 1,1,1, 0, 1, 0)
				GameTooltip:AddLine(' '..desc, 1,1,1, true)
				GameTooltip:AddLine(' ')
			end
			
			if relicLink3 then
				local quality, ilvl,_, _, _, _, _, texture = select(3, GetItemInfo(relicLink3))
				local spellID = ns.SpellIDByRelic[itemID..'.'..ns.GetItemID(relicLink3)]
				local spellName = GetSpellInfo(spellID)
				local desc = GetSpellDescription(spellID)
				local relicType = (select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink3))))
				local r, g, b, hex = GetItemQualityColor(quality)
				
				GameTooltip:AddDoubleLine('\124T'..texture..':22:22:0:0:64:64:4:60:4:60\124t'..'|c'..hex..'('..ilvl..')|r'..spellName, ns.RelicTypes[relicType], 1,1,1, 0, 1, 0)
				GameTooltip:AddLine(' '..desc, 1,1,1, true)
			end
			]==]
		else
			GameTooltip:AddLine(L['Restart client for correct work'], 1,0,0, true)
		end
		--[==[
		print('R1', relicLink1, ns.GetItemID(relicLink1), ns.SpellIDByRelic[itemID..'.'..ns.GetItemID(relicLink1)], (select(4, GetItemInfo(relicLink1))), 
			(select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink1)))))
			
			
		print('R2', relicLink2, ns.GetItemID(relicLink2), ns.SpellIDByRelic[itemID..'.'..ns.GetItemID(relicLink2)], (select(4, GetItemInfo(relicLink2))),
			(select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink2)))))
			
		print('R3', relicLink3, ns.GetItemID(relicLink3), ns.SpellIDByRelic[itemID..'.'..ns.GetItemID(relicLink3)], (select(4, GetItemInfo(relicLink3))),
			(select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink3)))))
		]==]
		
		GameTooltip:Show()
	end
	
	local function SetItemTip(frame)
		if not frame.link then return end
		
		frame:RegisterEvent('MODIFIER_STATE_CHANGED')
		
		GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMLEFT")
		
		local itemID = ns.GetItemID(frame.link)
		local tokenList = tierTokenItemList[itemID]
		local classIndex, specID, classFileName = ns.GetPlayerClassInfo()
		
		if ns.Artifacts[itemID] then
			
			local name = GetItemInfo(itemID)
			
			if name then
				ShowArtifactTooltip(itemID, frame.link)
			else
				C_Timer.After(0.3, function()
					ShowArtifactTooltip(itemID, frame.link)
				end)
			end
		elseif ns.db.EnableTokenItemHook and tokenList and tokenList[classFileName] then
			local bonus = 0
			if IsMythicToken(frame.link) then
				bonus = 566
			elseif IsHeroicToken(frame.link) then
				bonus = 567
			end
			
			local itemLink = format(linkPattern, tokenList[classFileName], bonus)
			
			GameTooltip:SetHyperlink(itemLink, classIndex, specID)
		elseif frame.classID and frame.specID then		
			GameTooltip:SetHyperlink(frame.link, frame.classID, frame.specID)
		else
		
			local checkForClass = ns.CheckForContainTooltip(frame.link, ns.classSpecificItem)

			if checkForClass then
				local amount, list = ns.GetClassList(checkForClass)
				
				local classTag, classIndexed = next(list)
			
				if classIndexed then				
					local specID, specName = GetSpecializationInfoForClassID(classIndexed, 1);
					
					GameTooltip:SetHyperlink(frame.link, classIndexed, specID)
				else
					GameTooltip:SetHyperlink(frame.link, classIndex, specID)
				end
			else
				GameTooltip:SetHyperlink(frame.link, classIndex, specID)
			end
		end
		
		if IsArtifactRelicItem(itemID) then
			ns.GetRelicClassSpec(itemID, GameTooltip)
		end
		
	--	GameTooltip:AddLine(("Diffic %s"):format( IsMythicToken(frame.link) or  IsHeroicToken(frame.link) or 'Обычный'))
		
		if IsShiftKeyDown() then 
			GameTooltip_ShowCompareItem() 
		end
		if IsModifiedClick("DRESSUP") then ShowInspectCursor() else ResetCursor() end
	end
 
	
	local function ItemOnUpdate(self)
		if IsShiftKeyDown() then 
		--	GameTooltip_ShowCompareItem() 
		end
	--	CursorOnUpdate(self)
	end
	local function LootClick(frame)
		if IsControlKeyDown() then DressUpItemLink(frame.link)
		elseif IsShiftKeyDown() then ChatEdit_InsertLink(frame.link) end
	end
	local function SetTip(frame)
		GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
		GameTooltip:SetText(frame.tiptext)
	end

	ns.HideTip = HideTip
	ns.HideTip2 = HideTip2
	ns.SetItemTip = SetItemTip
	ns.ItemOnUpdate = ItemOnUpdate
	ns.LootClick = LootClick
	ns.SetTip = SetTip
	
	
	function ns.GameTooltip_OnTooltipSetItem(self)
		local _, link = self:GetItem()
		if not link then 
			ns.HideRelicInfoFrame()
		--	print('HideRelicInfoFrame')
			return; 
		end
		
		local itemString = string.match(link, "item[%-?%d:]+")
		
		if itemString then
			local _, itemID = strsplit(":", itemString)
	
		
			if IsArtifactRelicItem(itemID) then
				ns.GetRelicClassSpec(itemID, self)
			--	print('GetRelicClassSpec')
			else
				ns.HideRelicInfoFrame()
		--		print('HideRelicInfoFrame')
			end
		else
			ns.HideRelicInfoFrame()
		--	print('HideRelicInfoFrame')
		end
	end
end

do
	local specList = {250,251,252,102,103,104,105,253,254,255,62,63,64,268,269,270,65,66,70,256,257,258,259,260,261,262,263,264,265,266,267,71,72,73, 577, 581}
	local specIcons = {}
	local specRole = {}

	for i=1, #specList do
		local id = specList[i]
		local id, name, description, icon, role, class = GetSpecializationInfoByID(id)
		specIcons[id] = icon
		specRole[id] = role
	end

	function ns.GetSpecClassString(classID, spec)
		if spec and specIcons[spec] then
			return specIcons[spec]
		elseif classID and GetClassInfoByID(classID) and CLASS_ICON_TCOORDS[select(2, GetClassInfoByID(classID))] then
			local t = CLASS_ICON_TCOORDS[select(2, GetClassInfoByID(classID))]
			return "Interface\\TargetingFrame\\UI-Classes-Circles", t[1], t[2], t[3], t[4]
		end
		
		return 'Interface\\Icons\\Inv_misc_questionmark'
	end
	
	function ns.GetSpecRole(spec)
		if spec and specRole[spec] then
			return specRole[spec]
		end
		
		return false
	end

end

do

	
	local function customGSUBHandler(pre, itemname, post)
		return pre..'L'..post
	end
	
	
	function ns.ShortLink(link)
		link = link:gsub('|c%x%x%x%x%x%x%x%x', '')
		link = link:gsub('|r', '')
		link = link:gsub("(|Hitem:.-|h%[)(.-)(%]|h)", customGSUBHandler)
		
		return link
	end
	
	function ns.NumericItemLink(link, from)
		local numer = false
		
		if link then
			numer = link:match('|Hitem:(.-)|h%[')
		else
			print('Error on NumericItemLink:', link, from)
		end
		
		return numer
	end
	
	
	local function customGSUBHandler2(pre, itemname, post)
		local name = ns:utf8sub(itemname, 1, 12)
		return pre..name..'...'..post
	end
	
	function ns.ShortLinkName(link)
		if ns.db.EnableShornLootLink then
			link = link:gsub("(|Hitem:.-|h%[)(.-)(%]|h)", customGSUBHandler2)
		end
		
		return link
	end
end

do
	local date = date
	local tremove = table.remove
	local tonumber = tonumber
	local strsplit = strsplit
	local tinsert = table.insert
	local format = format
	
	local timeOut = 60 * 60 * 24 *30
	
	local pattern = '%d\n%s\n%s\n%s\nboss:%d\nzone:%d\n%s\n%d\n%s'
	
	local date_pattern = "%m/%d/%y"
	local time_pattern = "%H:%M:%S"
	local full_patterm = date_pattern..' - '..time_pattern
	
	
	function ns.Histoty_FormatDate(timestamp)
		return date(full_patterm, timestamp)
	end
	
	function ns.Histoty_FormatDate_Month(timestamp)
		return tonumber(date('%m', timestamp))
	end
	
	function ns.Histoty_FormatDate_MDY(timestamp)
		return date(date_pattern, timestamp)
	end
	
	function ns.History_ReceiveData(info)
	
		if not ns.db.EnableLootRollLog then return end
		
		local lootData = { strsplit('\n', info) }
	
		local name       = lootData[1]
		local class      = lootData[2]
		local link       = lootData[3]
		local buttonID   = lootData[4]
		local randIndex  = lootData[5]
		local bossID	 = lootData[6]
		
	--	print('T', lootData[7], lootData[8], lootData[9])
		
		local rollID 	 = nil 
		
		if lootData[7] and lootData[8] and lootData[9] then
			rollID = lootData[7]..'\n'..lootData[8]..'\n'..lootData[9]
		end
	
		local buttonName, buttonTip, buttonArg 
		local buttonColor = '|cFFFFFFFF'
		local rollText = ''
		if buttonID and tonumber(buttonID) then
			buttonName, buttonTip, buttonArg = ns:GetButtonIDInfo(tonumber(buttonID))
			buttonColor = ns.rollTypeColors[buttonTip] or '|cFFFFFFFF'
		end
		
		if buttonName then
		
			rollText = buttonColor..buttonName..'|r'
			
			if randIndex then
				local tempRand = tonumber(randIndex)
				
				if tempRand and tempRand > 0 then				
					rollText = rollText..'(бросок:'..tempRand..')'
				end
			end
		end
		
		if ns:IsMasterLooter() then
			print('LH', ns.ShortLinkName(link), '->',ns.GetColoredNameByClassFile(name, class), '(|cFF00FF00'..L['ITEM_AWARDED']..'|r)', rollText)
		elseif ns.db.EnableAwardingLoot then
			print('LH', ns.ShortLinkName(link), '->',ns.GetColoredNameByClassFile(name, class), '(|cFF00FF00'..L['ITEM_AWARDED']..'|r)', rollText)
		end		
		--[==[
		if ns.showLootAnonce then
			print('LH', ns.ShortLinkName(link), '->',ns.GetColoredNameByClassFile(name, class), '(|cFF00FF00'..'Передано'..'|r)', rollText)
		end
		]==]
		ns.dprint('LH_3', ns.ShortLinkName(link), '->',ns.GetColoredNameByClassFile(name, class), '(|cFF00FF00'..L['ITEM_AWARDED']..'|r)', rollText)
		
		ns.History_AddLoot(name, link, class, buttonName, randIndex, bossID, rollID)
	end
	
	function ns.History_AddLoot(unit, lootlink, class, buttonName, randIndex, bossID, rollID)
		local timestamp = time()
		
		local data = ns.Histoty_FormatDate_MDY(timestamp)
	
		ns.db.lootHistory_Date[data] = timestamp
		ns.db.lootHistory_Units[unit] = class or true
		
		if not ns.db.lootHistory_Date_Units[data] then
			ns.db.lootHistory_Date_Units[data] = {}
		end
		ns.db.lootHistory_Date_Units[data][unit] = class or true
	
		local instanceID = ns.Cache_GetInstanceByBossID(bossID)
		
		if bossID and instanceID then
			if not ns.db.lootHistory_Encounter_Boss[instanceID] then
				ns.db.lootHistory_Encounter_Boss[instanceID] = {}
			end			
			ns.db.lootHistory_Encounter_Boss[instanceID][bossID] = bossID
		else
			bossID, instanceID = ns.Cache_GetItemIDInfo(ns.GetItemID(lootlink))
			
			if bossID and instanceID then
				if not ns.db.lootHistory_Encounter_Boss[instanceID] then
					ns.db.lootHistory_Encounter_Boss[instanceID] = {}
				end			
				ns.db.lootHistory_Encounter_Boss[instanceID][bossID] = bossID
			end
		end
		
		local str = format(pattern, timestamp, ns.Histoty_FormatDate(timestamp), unit, lootlink, bossID or 0, instanceID or 0, buttonName or UNKNOWN, randIndex or -1, ns:ExportRollIDToString(rollID))
		
		tinsert(ns.db.lootHistory, str)
		
		ns.LootList_ResetPosition()
	end

	
	function ns.History_AddLootRandom()
		-- "\124cffa335ee\124Hitem:124382:0:0:0:0:0:0:0:0:0:529:0:4:41:562:565:567\124h[Edict of Argus]\124h\124r"
		local link = "\124cffa335ee\124Hitem:124382:0:0:0:0:0:0:0:0:0:529:0:4:41:562:565:567\124h[Edict of Argus]\124h\124r"
		local unit = 'unit'..fastrandom(1,15)
			unit = unit..'-Гордунни'..fastrandom(1,25)
			
		local timespamp = time()+fastrandom(-50, 50)
		local class = fastrandom(1, 10)
		
		local tDate = ns.Histoty_FormatDate_MDY(timestamp)
		
		ns.db.lootHistory_Date[tDate] = timespamp
		ns.db.lootHistory_Units[unit] = class
		
		if not ns.db.lootHistory_Date_Units[tDate] then
			ns.db.lootHistory_Date_Units[tDate] = {}
		end
		ns.db.lootHistory_Date_Units[tDate][unit] = class
		
		local bossID, instanceID = ns.Cache_GetItemIDInfo(ns.GetItemID(link))
		
		if instanceID and bossID then
			if not ns.db.lootHistory_Encounter_Boss[instanceID] then
				ns.db.lootHistory_Encounter_Boss[instanceID] = {}
			end			
			ns.db.lootHistory_Encounter_Boss[instanceID][bossID] = bossID
		end
		
		local str = format(pattern, timespamp, ns.Histoty_FormatDate(timestamp), unit, link, bossID or 0, instanceID or 0)
		
		tinsert(ns.db.lootHistory, str)
	end
	
	function ns.History_Reset()
		
		ns.db.lootHistory_Date = {}
		ns.db.lootHistory_Units = {}
		ns.db.lootHistory_Date_Units = {}
		ns.db.lootHistory = {}
		ns.db.lootHistory_Encounter_Boss = {}
		
	end
	
	local function ScanForClear(timestamp)
		local index

		while ns.db.lootHistory[index] do
			local timestamp2 = strsplit('\n', ns.db.lootHistory[i])
			
			if ( timestamp - tonumber(timestamp2) ) > timeOut then
				tremove(ns.db.lootHistory, index);
				coroutine.yield();
			else
				index = index + 1;
			end	
		end
	end
	
	function ns.Histort_PostClearPrep(timestamp)
		local newUnitList = {}
		
		for tDate, timespamp1 in pairs(ns.db.lootHistory_Date) do
		
		--	print('T', tDate, ( timestamp - timespamp1 ) > timeOut)
			
			if ( timestamp - timespamp1 ) > timeOut then	

				for nameServer in pairs(ns.db.lootHistory_Date_Units[tDate]) do				
					ns.db.lootHistory_Units[nameServer] = nil
				end
				
				ns.db.lootHistory_Date_Units[tDate] = nil
				ns.db.lootHistory_Date[tDate] = nil
			else
				for nameServer, class in pairs(ns.db.lootHistory_Date_Units[tDate]) do			
					newUnitList[nameServer] = class
				end
			end
		end
		
		local missedUnits = 0
		
		for nameServer, class in pairs(ns.db.lootHistory_Units) do
			if class and not newUnitList[nameServer] then
				missedUnits = missedUnits + 1
				ns.db.lootHistory_Units[nameServer] = nil
			end
		end
		
		for nameServer, class in pairs( newUnitList ) do		
			ns.db.lootHistory_Units[nameServer] = class
		end
		
		wipe(newUnitList)
	end
	
	function ns.History_AutoClear()
		local func = function()
			local timestamp = time()
			ScanForClear(timestamp)
			ns.Histort_PostClearPrep(timestamp)
		end

		local co = coroutine.create(func);
		ns.dynFrame.AddAction("l", co);
	end
end

do
	local pairs = pairs
	
	function ns.CompareTables(t1, t2)
		local t1_num = #t1

		local sameValues = 0
		local containSame = false
		
		for i, value in pairs(t1) do		
			for i2, value2 in pairs(t2) do				
				if value == value2 then
					sameValues = sameValues + 1
					containSame = true
				end
			end
		end
		
		
		return t1_num == sameValues, containSame
	end

end

do
	local LootHelperCache = {
		["encounters"] = {
			[1202] = 457,
			[1203] = 457,
			[959] = 457,
			[1391] = 669,
			[1392] = 669,
			[1425] = 669,
			[1426] = 669,
			[1427] = 669,
			[1432] = 669,
			[1433] = 669,
			[1372] = 669,
			[1147] = 457,
			[1438] = 669,
			[1154] = 457,
			[1161] = 457,
			[1162] = 457,
			[1123] = 457,
			[1396] = 669,
			[1122] = 457,
			[1155] = 457,
			[1394] = 669,
			[1447] = 669,
			[1395] = 669,
			
			-- Legion
			-- Emerald Nightmare
			[1744] = 768,
			[1703] = 768,
			[1667] = 768,
			[1704] = 768,
			[1750] = 768,
			[1726] = 768,
			[1738] = 768,
			
			-- Nighthold
			[1737] = 786,
			[1761] = 786,
			[1743] = 786,
			[1713] = 786,
			[1762] = 786,
			[1706] = 786,
			[1725] = 786,
			[1751] = 786,
			[1731] = 786,
			[1732] = 786,
			
			-- Trial of Valor
			[1830] = 861,
			[1819] = 861,
			[1829] = 861,
			
			-- Гробница
			[1861] = 875,
			[1873] = 875,
			[1862] = 875,
			[1896] = 875,
			[1897] = 875,
			[1856] = 875,
			[1867] = 875,
			[1898] = 875,
			[1903] = 875,
			
			-- Анторус
			[1992] = 946,
			[1985] = 946,
			[2004] = 946,
			[1997] = 946,
			[1986] = 946,
			[2009] = 946,
			[1983] = 946,
			[1987] = 946,
			[2025] = 946,
			[1984] = 946,
			[2031] = 946,
		},
		["items"] = {
			[124169] = 1396,
			[124201] = 1426,
			[124233] = 1427,
			[124265] = 1391,
			[124297] = 1392,
			[124329] = 1427,
			[124361] = 1395,
			[119315] = 1123,
			[124521] = 1438,
			[113886] = 1154,
			[124138] = 1391,
			[124170] = 1433,
			[124202] = 1433,
			[124234] = 1447,
			[124266] = 1395,
			[124298] = 1426,
			[124330] = 1396,
			[124362] = 1427,
			[119316] = 959,
			[124522] = 1438,
			[113887] = 1154,
			[124139] = 1392,
			[124171] = 1395,
			[124203] = 1391,
			[124235] = 1392,
			[124267] = 1372,
			[124299] = 1432,
			[124331] = 1427,
			[124363] = 1392,
			[124523] = 1438,
			[113888] = 1154,
			[124140] = 1372,
			[127749] = 1433,
			[124204] = 1395,
			[124236] = 1372,
			[124268] = 1372,
			[124300] = 1394,
			[124332] = 1392,
			[124364] = 1396,
			[119318] = 1123,
			[113889] = 1154,
			[124141] = 1427,
			[124173] = 1395,
			[124205] = 1372,
			[124237] = 1391,
			[124269] = 1372,
			[124301] = 1372,
			[124333] = 1392,
			[124365] = 1426,
			[119319] = 1203,
			[113890] = 1154,
			[124142] = 1432,
			[124174] = 1425,
			[124206] = 1395,
			[124238] = 1395,
			[124270] = 1426,
			[124302] = 1372,
			[124334] = 1392,
			[124366] = 1396,
			[119320] = 1154,
			[119448] = 1202,
			[113891] = 1154,
			[124143] = 1427,
			[124175] = 1433,
			[124207] = 1426,
			[124239] = 1392,
			[124271] = 1432,
			[124303] = 1395,
			[124335] = 1426,
			[124367] = 1395,
			[119321] = 1162,
			[113892] = 1154,
			[124144] = 1438,
			[124176] = 1438,
			[124208] = 1396,
			[124240] = 1396,
			[124272] = 1447,
			[124304] = 1392,
			[124336] = 1432,
			[124368] = 1391,
			[119322] = 1147,
			[113893] = 1154,
			[124145] = 1425,
			[124177] = 1447,
			[124209] = 1391,
			[124241] = 1433,
			[124273] = 1447,
			[124305] = 1396,
			[124337] = 1391,
			[124369] = 1391,
			[123890] = 1438,
			[119323] = 959,
			[113862] = 1161,
			[113894] = 1154,
			[124146] = 1372,
			[124178] = 1447,
			[124210] = 1438,
			[124242] = 1394,
			[124274] = 1447,
			[124306] = 1394,
			[124338] = 1372,
			[124370] = 1426,
			[113863] = 1161,
			[113895] = 1154,
			[124147] = 1447,
			[124179] = 1447,
			[124211] = 1392,
			[124243] = 1392,
			[124275] = 1433,
			[124307] = 1447,
			[124339] = 1372,
			[124371] = 1447,
			[113864] = 1161,
			[113896] = 1154,
			[124148] = 1425,
			[124180] = 1392,
			[124212] = 1447,
			[124244] = 1372,
			[124276] = 1438,
			[124308] = 1447,
			[124340] = 1372,
			[124372] = 1438,
			[113865] = 1161,
			[113897] = 1155,
			[124149] = 1391,
			[124181] = 1391,
			[124213] = 1425,
			[124245] = 1427,
			[124309] = 1425,
			[124341] = 1392,
			[124373] = 1425,
			[113866] = 1161,
			[113898] = 1155,
			[113930] = 1162,
			[113962] = 1147,
			[124214] = 1391,
			[124246] = 1395,
			[124278] = 1426,
			[124310] = 1391,
			[124342] = 1372,
			[124374] = 1447,
			[113867] = 1161,
			[113899] = 1155,
			[124151] = 1392,
			[124183] = 1426,
			[124215] = 1395,
			[124247] = 1395,
			[124279] = 1396,
			[124343] = 1438,
			[124375] = 1394,
			[113868] = 1161,
			[113900] = 1155,
			[124152] = 1396,
			[124184] = 1432,
			[124216] = 1426,
			[124248] = 1395,
			[124280] = 1391,
			[124312] = 1432,
			[124344] = 1447,
			[124376] = 1394,
			[113869] = 1161,
			[113901] = 1155,
			[124153] = 1394,
			[124185] = 1427,
			[124217] = 1392,
			[124249] = 1425,
			[124281] = 1396,
			[124313] = 1427,
			[124345] = 1447,
			[124377] = 1392,
			[113987] = 959,
			[113986] = 959,
			[113984] = 959,
			[113983] = 959,
			[113870] = 1161,
			[113902] = 1155,
			[124154] = 1427,
			[124186] = 1394,
			[124218] = 1433,
			[124250] = 1396,
			[124282] = 1433,
			[124314] = 1395,
			[124346] = 1447,
			[124378] = 1395,
			[113985] = 959,
			[113989] = 959,
			[113982] = 959,
			[113990] = 959,
			[113981] = 959,
			[113980] = 959,
			[113871] = 1161,
			[113903] = 1155,
			[124155] = 1427,
			[124187] = 1392,
			[124219] = 1394,
			[124251] = 1394,
			[124283] = 1391,
			[124315] = 1425,
			[124347] = 1432,
			[124379] = 1396,
			[113979] = 959,
			[113988] = 959,
			[113925] = 1123,
			[116660] = 959,
			[113969] = 1203,
			[113975] = 1203,
			[113872] = 1161,
			[113904] = 1155,
			[124156] = 1427,
			[124188] = 1372,
			[124220] = 1432,
			[113974] = 1203,
			[124284] = 1395,
			[124316] = 1433,
			[124348] = 1372,
			[124380] = 1372,
			[113970] = 1203,
			[113976] = 1203,
			[113967] = 1203,
			[113968] = 1203,
			[113977] = 1203,
			[113972] = 1203,
			[113873] = 1161,
			[113905] = 1155,
			[124157] = 1372,
			[124189] = 1447,
			[124221] = 1427,
			[124253] = 1425,
			[124285] = 1425,
			[124317] = 1395,
			[124349] = 1395,
			[124381] = 1447,
			[113971] = 1203,
			[113978] = 1203,
			[113966] = 1203,
			[113973] = 1203,
			[113965] = 1203,
			[120077] = 1123,
			[113874] = 1202,
			[113906] = 1155,
			[124158] = 1447,
			[124190] = 1426,
			[124222] = 1438,
			[124254] = 1438,
			[124286] = 1433,
			[124318] = 1395,
			[120078] = 1161,
			[124382] = 1438,
			[113931] = 1162,
			[113938] = 1162,
			[113936] = 1162,
			[113933] = 1162,
			[113935] = 1162,
			[113937] = 1162,
			[113875] = 1202,
			[113907] = 1155,
			[124159] = 1438,
			[124191] = 1427,
			[124223] = 1425,
			[124255] = 1427,
			[124287] = 1394,
			[124319] = 1395,
			[124351] = 1425,
			[124383] = 1432,
			[119305] = 1123,
			[113929] = 1162,
			[113928] = 1162,
			[113932] = 1162,
			[113926] = 1162,
			[113934] = 1162,
			[113876] = 1202,
			[113908] = 1155,
			[124160] = 1392,
			[124192] = 1394,
			[124224] = 1432,
			[124256] = 1427,
			[113927] = 1162,
			[124320] = 1426,
			[124352] = 1433,
			[124384] = 1427,
			[119306] = 1203,
			[113958] = 1147,
			[119194] = 1202,
			[113942] = 1122,
			[113954] = 1147,
			[113957] = 1147,
			[113877] = 1202,
			[113963] = 1147,
			[124161] = 1392,
			[124193] = 1432,
			[124225] = 1427,
			[124257] = 1427,
			[124289] = 1426,
			[124321] = 1396,
			[124353] = 1447,
			[124385] = 1432,
			[119307] = 1154,
			[113961] = 1147,
			[113964] = 1147,
			[124513] = 1438,
			[113955] = 1147,
			[113956] = 1147,
			[113878] = 1202,
			[113910] = 1155,
			[124162] = 1392,
			[124194] = 1433,
			[124226] = 1394,
			[124258] = 1432,
			[124290] = 1372,
			[124322] = 1438,
			[124354] = 1425,
			[124386] = 1427,
			[119308] = 1162,
			[113959] = 1147,
			[113960] = 1147,
			[124514] = 1438,
			[113953] = 1147,
			[113951] = 1122,
			[113879] = 1202,
			[113922] = 1123,
			[124163] = 1432,
			[124195] = 1394,
			[124227] = 1425,
			[124259] = 1433,
			[124291] = 1438,
			[119193] = 1123,
			[124355] = 1394,
			[124387] = 1433,
			[119309] = 1147,
			[113917] = 1123,
			[113919] = 1123,
			[124515] = 1438,
			[113914] = 1123,
			[113921] = 1123,
			[113880] = 1202,
			[124132] = 1426,
			[124164] = 1395,
			[124196] = 1425,
			[124228] = 1432,
			[124260] = 1447,
			[124292] = 1427,
			[124324] = 1426,
			[124356] = 1426,
			[124388] = 1432,
			[119310] = 959,
			[113915] = 1123,
			[113916] = 1123,
			[124516] = 1438,
			[113924] = 1123,
			[113923] = 1123,
			[113881] = 1202,
			[124133] = 1432,
			[124165] = 1372,
			[124197] = 1396,
			[124229] = 1433,
			[124261] = 1392,
			[124293] = 1427,
			[124325] = 1396,
			[124357] = 1427,
			[124389] = 1438,
			[119311] = 1203,
			[113920] = 1123,
			[113918] = 1123,
			[124517] = 1438,
			[113913] = 1123,
			[118114] = 1161,
			[113882] = 1202,
			[124134] = 1433,
			[124166] = 1372,
			[124198] = 1433,
			[124230] = 1395,
			[124262] = 1392,
			[124294] = 1433,
			[124326] = 1394,
			[124358] = 1392,
			[124390] = 1433,
			[119312] = 1162,
			[113948] = 1122,
			[119192] = 1122,
			[124518] = 1438,
			[113940] = 1122,
			[113947] = 1122,
			[113883] = 1202,
			[124135] = 1395,
			[124167] = 1372,
			[124199] = 1447,
			[124231] = 1426,
			[124263] = 1392,
			[124295] = 1447,
			[124327] = 1427,
			[124359] = 1372,
			[124391] = 1372,
			[119313] = 1154,
			[113944] = 1122,
			[113941] = 1122,
			[124519] = 1438,
			[113950] = 1122,
			[113949] = 1122,
			[113884] = 1202,
			[124136] = 1426,
			[124168] = 1426,
			[124200] = 1438,
			[124232] = 1396,
			[124264] = 1425,
			[124296] = 1392,
			[124328] = 1427,
			[124360] = 1391,
			[113943] = 1122,
			[119314] = 1147,
			[113945] = 1122,
			[113952] = 1122,
			[124520] = 1438,
			[113946] = 1122,
			[113939] = 1122,
			[113885] = 1154,
			[124137] = 1396,
			[124172] = 1395,
	
			[127959] = 1392, --Кормрок
			[127966] = 1392, --Кормрок
			[127956] = 1392, --Кормрок
			
			[127955] = 1372, --Кровожад
			[127960] = 1372, --Кровожад
			[127965] = 1372, --Кровожад
			
			[127954] = 1427, --Сокретар
			[127958] = 1427, --Сокретар
			[127964] = 1427, --Сокретар
			
			[127967] = 1447, --Ксул'горак
			[127961] = 1447, --Ксул'горак
			[127957] = 1447, --Ксул'горак
			
			[127962] = 1395, --Маннорот
			[127963] = 1395, --Маннорот
			[127953] = 1395, --Маннорот
			
			-- Legion
			-- Emerald Nightmare
			[139262] = 1667,
			[139326] = 1738,
			[139215] = 1738,
			[138215] = 1704,
			[139200] = 1703,
			[139216] = 1726,
			[138216] = 1704,
			[139248] = 1750,
			[139264] = 1750,
			[139201] = 1667,
			[139233] = 1738,
			[139249] = 1703,
			[139265] = 1750,
			[139329] = 1667,
			[139234] = 1667,
			[139250] = 1703,
			[139266] = 1726,
			[139187] = 1738,
			[139330] = 1667,
			[139219] = 1744,
			[139235] = 1703,
			[139251] = 1703,
			[139267] = 1726,
			[139188] = 1738,
			[139204] = 1744,
			[139220] = 1667,
			[139236] = 1703,
			[139252] = 1703,
			[139268] = 1726,
			[139335] = 1726,
			[139336] = 1726,
			[139189] = 1726,
			[139205] = 1704,
			[139221] = 1744,
			[139237] = 1738,
			[139253] = 1744,
			[139269] = 1726,
			[139238] = 1726,
			[139199] = 1726,
			[139190] = 1744,
			[139206] = 1738,
			[139222] = 1726,
			[138222] = 1704,
			[139254] = 1744,
			[139203] = 1726,
			[139228] = 1726,
			[139191] = 1703,
			[139207] = 1750,
			[139193] = 1726,
			[139239] = 1667,
			[139255] = 1744,
			[139232] = 1726,
			[139332] = 1726,
			[138827] = 1726,
			[139192] = 1750,
			[139208] = 1667,
			[139224] = 1703,
			[138224] = 1704,
			[139256] = 1738,
			[139333] = 1750,
			[139334] = 1750,
			[139320] = 1703,
			[139209] = 1744,
			[139225] = 1750,
			[138225] = 1704,
			[139257] = 1738,
			[139194] = 1750,
			[138211] = 1704,
			[139198] = 1750,
			[139321] = 1703,
			[139218] = 1750,
			[139226] = 1667,
			[138226] = 1704,
			[139258] = 1738,
			[139231] = 1750,
			[139263] = 1750,
			[138220] = 1704,
			[139195] = 1667,
			[139211] = 1750,
			[139227] = 1738,
			[138227] = 1704,
			[139259] = 1738,
			[138214] = 1704,
			[138219] = 1704,
			[138218] = 1704,
			[139323] = 1744,
			[139212] = 1703,
			[138212] = 1704,
			[138228] = 1704,
			[139260] = 1667,
			[139328] = 1667,
			[139327] = 1667,
			[139322] = 1744,
			[139324] = 1738,
			[139213] = 1738,
			[139229] = 1744,
			[138217] = 1744,
			[139261] = 1667,
			[139196] = 1667,
			[139217] = 1667,
			[139202] = 1738,
			[139325] = 1738,
			[139214] = 1703,
			[139230] = 1744,
			[138221] = 1744,
			[139197] = 1703,
			
			-- Nighthold
			[140818] = 1731,
			[140850] = 1751,
			[140882] = 1725,
			[138364] = 1731,
			[140819] = 1762,
			[140851] = 1731,
			[140883] = 1761,
			[138365] = 1731,
			[140820] = 1737,
			[140852] = 1751,
			[140884] = 1706,
			[138366] = 1731,
			[140789] = 1706,
			[140821] = 1725,
			[140853] = 1725,
			[140885] = 1732,
			[138367] = 1731,
			[140790] = 1706,
			[140822] = 1761,
			[140854] = 1731,
			[138336] = 1762,
			[138368] = 1731,
			[140791] = 1725,
			[140823] = 1737,
			[140855] = 1762,
			[138337] = 1762,
			[138369] = 1731,
			[140792] = 1725,
			[140824] = 1762,
			[140888] = 1706,
			[138370] = 1731,
			[140793] = 1731,
			[140825] = 1713,
			[140857] = 1713,
			[138339] = 1713,
			[138371] = 1731,
			[140794] = 1731,
			[140826] = 1737,
			[140858] = 1731,
			[138340] = 1732,
			[138372] = 1731,
			[140795] = 1751,
			[140827] = 1706,
			[140859] = 1762,
			[138341] = 1732,
			[138373] = 1731,
			[140796] = 1751,
			[140860] = 1725,
			[140892] = 1762,
			[138374] = 1731,
			[140797] = 1762,
			[140861] = 1761,
			[138343] = 1743,
			[138375] = 1731,
			[140798] = 1762,
			[140862] = 1706,
			[140894] = 1725,
			[138376] = 1713,
			[140799] = 1713,
			[140831] = 1725,
			[140863] = 1725,
			[140895] = 1751,
			[138377] = 1732,
			[140800] = 1713,
			[140832] = 1751,
			[140864] = 1732,
			[140896] = 1761,
			[138378] = 1743,
			[140801] = 1761,
			[140833] = 1732,
			[140865] = 1731,
			[138347] = 1762,
			[138379] = 1737,
			[140802] = 1761,
			[140834] = 1751,
			[140866] = 1751,
			[138348] = 1762,
			[138380] = 1762,
			[140803] = 1732,
			[140835] = 1713,
			[140867] = 1751,
			[138349] = 1713,
			[140893] = 1743,
			[140804] = 1732,
			[140836] = 1761,
			[138828] = 1725,
			[138350] = 1713,
			[140917] = 1737,
			[138358] = 1737,
			[140897] = 1737,
			[140914] = 1737,
			[140805] = 1743,
			[140837] = 1743,
			[138319] = 1713,
			[138351] = 1713,
			[138335] = 1737,
			[138345] = 1737,
			[138316] = 1737,
			[138344] = 1737,
			[140806] = 1743,
			[140838] = 1731,
			[138320] = 1713,
			[140902] = 1706,
			[138334] = 1737,
			[138315] = 1737,
			[138333] = 1737,
			[138317] = 1737,
			[140807] = 1737,
			[140839] = 1761,
			[138321] = 1762,
			[140903] = 1725,
			[138360] = 1737,
			[138359] = 1737,
			[140919] = 1737,
			[140913] = 1737,
			[140808] = 1737,
			[140840] = 1706,
			[138322] = 1762,
			[140904] = 1731,
			[138318] = 1713,
			[140890] = 1743,
			[140912] = 1743,
			[140889] = 1743,
			[140809] = 1737,
			[140841] = 1732,
			[140873] = 1761,
			[140905] = 1751,
			[138338] = 1762,
			[140911] = 1743,
			[138313] = 1743,
			[138356] = 1743,
			[140810] = 1743,
			[140842] = 1743,
			[138324] = 1713,
			[140906] = 1762,
			[138355] = 1743,
			[138332] = 1743,
			[138342] = 1743,
			[138331] = 1743,
			[138312] = 1743,
			[140843] = 1725,
			[140875] = 1706,
			[140907] = 1713,
			[138330] = 1743,
			[138314] = 1743,
			[138357] = 1743,
			[140891] = 1732,
			[140812] = 1731,
			[140844] = 1762,
			[140876] = 1706,
			[140908] = 1761,
			[138311] = 1732,
			[138352] = 1732,
			[138329] = 1732,
			[138354] = 1732,
			[140813] = 1751,
			[140845] = 1732,
			[140877] = 1751,
			[140909] = 1732,
			[138353] = 1732,
			[138310] = 1732,
			[138328] = 1732,
			[138309] = 1732,
			[138327] = 1732,
			[140898] = 1706,
			[140878] = 1751,
			[140910] = 1743,
			[140899] = 1713,
			[140872] = 1725,
			[138325] = 1713,
			[140868] = 1751,
			[140815] = 1706,
			[140900] = 1732,
			[140879] = 1725,
			[138361] = 1762,
			[138323] = 1762,
			[140869] = 1731,
			[140887] = 1713,
			[140901] = 1706,
			[140816] = 1713,
			[140848] = 1725,
			[140880] = 1731,
			[138362] = 1762,
			[140870] = 1713,
			[138346] = 1713,
			[140886] = 1761,
			[140871] = 1731,
			[140817] = 1737,
			[140849] = 1706,
			[140881] = 1761,
			[138363] = 1762,
			[140874] = 1761,
			[138326] = 1713,
			
			-- Trial of Valor
			[142412] = 1830,
			[142420] = 1829,
			[142428] = 1829,
			[142507] = 1829,
			[142515] = 1830,
			[142413] = 1830,
			[142421] = 1819,
			[142429] = 1829,
			[142508] = 1819,
			[142516] = 1830,
			[142540] = 1819,
			[142422] = 1819,
			[142430] = 1819,
			[142517] = 1829,
			[142415] = 1830,
			[142423] = 1830,
			[142431] = 1819,
			[142510] = 1829,
			[142518] = 1829,
			[142416] = 1829,
			[142424] = 1829,
			[142432] = 1830,
			[142511] = 1819,
			[142519] = 1819,
			[142417] = 1830,
			[142425] = 1819,
			[142433] = 1829,
			[142512] = 1830,
			[142520] = 1819,
			[142410] = 1819,
			[142418] = 1829,
			[142426] = 1830,
			[142434] = 1830,
			[142513] = 1829,
			[142521] = 1829,
			[142411] = 1819,
			[142419] = 1819,
			[142427] = 1829,
			[142435] = 1830,
			[142506] = 1830,
			[142514] = 1819,
			[142414] = 1829,
			
			-- Гробница
			[147097] = 1903,
			[147129] = 1856,
			[147161] = 1861,
			[147193] = 1873,
			[147002] = 1856,
			[147034] = 1898,
			[147066] = 1898,
			[147098] = 1897,
			[147130] = 1867,
			[147162] = 1873,
			[147194] = 1897,
			[147003] = 1867,
			[147035] = 1867,
			[147067] = 1856,
			[147099] = 1898,
			[147131] = 1861,
			[147163] = 1896,
			[147195] = 1898,
			[147004] = 1861,
			[147036] = 1897,
			[147068] = 1903,
			[147100] = 1856,
			[147132] = 1873,
			[147164] = 1856,
			[147005] = 1903,
			[147037] = 1856,
			[147069] = 1862,
			[147101] = 1903,
			[147133] = 1898,
			[147165] = 1867,
			[147006] = 1873,
			[147070] = 1896,
			[147102] = 1873,
			[147134] = 1896,
			[147166] = 1861,
			[147007] = 1898,
			[147039] = 1862,
			[147071] = 1856,
			[147135] = 1856,
			[147167] = 1898,
			[147040] = 1896,
			[147072] = 1898,
			[147104] = 1862,
			[147136] = 1867,
			[147168] = 1873,
			[147009] = 1862,
			[147041] = 1867,
			[147073] = 1867,
			[147105] = 1903,
			[147137] = 1861,
			[147169] = 1898,
			[147010] = 1867,
			[147042] = 1873,
			[147074] = 1897,
			[147106] = 1896,
			[147138] = 1873,
			[147170] = 1896,
			[147011] = 1861,
			[147043] = 1856,
			[147107] = 1873,
			[147139] = 1898,
			[147171] = 1856,
			[147012] = 1903,
			[147076] = 1862,
			[147108] = 1862,
			[147140] = 1896,
			[147172] = 1867,
			[147013] = 1867,
			[147045] = 1856,
			[147077] = 1867,
			[147109] = 1856,
			[147141] = 1856,
			[147173] = 1861,
			[147014] = 1896,
			[147046] = 1898,
			[147078] = 1903,
			[147110] = 1896,
			[147142] = 1867,
			[147174] = 1873,
			[147015] = 1897,
			[147047] = 1896,
			[147079] = 1873,
			[147111] = 1873,
			[147143] = 1861,
			[147175] = 1898,
			[146984] = 1862,
			[147016] = 1861,
			[147048] = 1898,
			[147080] = 1867,
			[147112] = 1862,
			[147144] = 1873,
			[147176] = 1896,
			[146985] = 1861,
			[147017] = 1903,
			[147049] = 1861,
			[147081] = 1861,
			[147113] = 1861,
			[147145] = 1896,
			[147177] = 1856,
			[146986] = 1862,
			[147018] = 1896,
			[147050] = 1897,
			[147082] = 1898,
			[147114] = 1896,
			[147146] = 1856,
			[147178] = 1867,
			[146987] = 1903,
			[147019] = 1898,
			[147051] = 1867,
			[147115] = 1897,
			[147147] = 1867,
			[147179] = 1861,
			[146988] = 1898,
			[147020] = 1856,
			[147052] = 1873,
			[147084] = 1862,
			[147148] = 1861,
			[147180] = 1873,
			[147021] = 1903,
			[147053] = 1867,
			[147085] = 1861,
			[147149] = 1898,
			[147181] = 1896,
			[146990] = 1861,
			[147022] = 1862,
			[147054] = 1903,
			[147086] = 1896,
			[147150] = 1873,
			[147182] = 1856,
			[146991] = 1873,
			[147023] = 1861,
			[147055] = 1862,
			[147087] = 1898,
			[147151] = 1898,
			[147183] = 1867,
			[146992] = 1862,
			[147024] = 1896,
			[147056] = 1903,
			[147088] = 1867,
			[147152] = 1896,
			[147184] = 1861,
			[146993] = 1897,
			[147025] = 1897,
			[147057] = 1862,
			[147089] = 1903,
			[147121] = 1898,
			[147153] = 1856,
			[147185] = 1898,
			[146994] = 1856,
			[147026] = 1898,
			[147058] = 1897,
			[147090] = 1897,
			[147122] = 1896,
			[147154] = 1867,
			[147186] = 1873,
			[146995] = 1896,
			[147027] = 1862,
			[147059] = 1861,
			[147091] = 1873,
			[147123] = 1856,
			[147155] = 1861,
			[147187] = 1898,
			[146996] = 1867,
			[147028] = 1861,
			[147060] = 1897,
			[147092] = 1856,
			[147124] = 1867,
			[147156] = 1873,
			[147188] = 1896,
			[146997] = 1903,
			[147029] = 1856,
			[147061] = 1903,
			[147093] = 1861,
			[147125] = 1861,
			[147157] = 1898,
			[147189] = 1856,
			[146998] = 1867,
			[147030] = 1873,
			[147062] = 1873,
			[147094] = 1897,
			[147126] = 1873,
			[147158] = 1896,
			[147190] = 1867,
			[146999] = 1897,
			[147031] = 1903,
			[147063] = 1896,
			[147095] = 1898,
			[147127] = 1898,
			[147159] = 1856,
			[147191] = 1861,
			[147000] = 1856,
			[147032] = 1897,
			[147096] = 1867,
			[147128] = 1896,
			[147160] = 1867,
			[147192] = 1873,
			[147001] = 1873,
			[147033] = 1903,
			[147065] = 1862,
			[147347] = 1898,
			
			-- анторус
			[151942] = 1983,
			[151974] = 1987,
			[152006] = 1997,
			[152293] = 1997,
			[152134] = 2009,
			[152166] = 2025,
			[151943] = 1992,
			[151975] = 2004,
			[152007] = 2025,
			[152039] = 1992,
			[152645] = 1987,
			[152167] = 1997,
			[151944] = 2009,
			[151976] = 1985,
			[152008] = 1985,
			[152295] = 1997,
			[152136] = 1997,
			[152168] = 2004,
			[151945] = 1985,
			[151977] = 1986,
			[152009] = 1992,
			[152041] = 1985,
			[152137] = 2004,
			[152424] = 1997,
			[151946] = 1986,
			[151978] = 1984,
			[152010] = 1986,
			[152042] = 2009,
			[152138] = 1984,
			[152170] = 2009,
			[151947] = 1987,
			[151979] = 1983,
			[152011] = 1997,
			[152043] = 1997,
			[152139] = 2009,
			[152681] = 2025,
			[151948] = 2004,
			[151980] = 1987,
			[152012] = 1987,
			[152044] = 2025,
			[152140] = 2025,
			[152172] = 1997,
			[151949] = 1987,
			[151981] = 2025,
			[152013] = 2025,
			[152045] = 2009,
			[152141] = 1986,
			[152173] = 2004,
			[151950] = 1984,
			[151982] = 2031,
			[152014] = 2004,
			[152046] = 1986,
			[152142] = 2025,
			[152174] = 1984,
			[151951] = 1992,
			[151983] = 1987,
			[152015] = 1983,
			[152047] = 2025,
			[152143] = 1997,
			[152175] = 2009,
			[151952] = 2025,
			[151984] = 1986,
			[152016] = 2031,
			[152048] = 2004,
			[152112] = 2025,
			[152144] = 2004,
			[152176] = 2025,
			[151953] = 1997,
			[151985] = 1997,
			[152017] = 1992,
			[152049] = 1985,
			[152113] = 1997,
			[152145] = 1984,
			[152687] = 2009,
			[151954] = 1983,
			[151986] = 2031,
			[152018] = 1984,
			[152050] = 2009,
			[152114] = 2004,
			[152146] = 2009,
			[152688] = 2025,
			[151955] = 2004,
			[151987] = 1992,
			[152019] = 1997,
			[152816] = 1987,
			[152115] = 1984,
			[152147] = 1986,
			[152179] = 1997,
			[151956] = 1992,
			[151988] = 1992,
			[152020] = 1985,
			[152052] = 1984,
			[152116] = 2009,
			[152148] = 2025,
			[152180] = 2004,
			[151957] = 1997,
			[151989] = 2009,
			[152021] = 1987,
			[152117] = 1986,
			[152149] = 1997,
			[152181] = 1984,
			[151958] = 1985,
			[151990] = 1985,
			[152022] = 1984,
			[152054] = 2025,
			[152086] = 1985,
			[152118] = 2025,
			[152150] = 2004,
			[152182] = 2009,
			[151991] = 1983,
			[152023] = 2025,
			[152055] = 2004,
			[152119] = 1997,
			[152151] = 1984,
			[152183] = 1986,
			[151960] = 1983,
			[151992] = 1997,
			[152024] = 2009,
			[152056] = 1987,
			[152120] = 2004,
			[152152] = 2009,
			[152280] = 2004,
			[152057] = 1985,
			[152121] = 1984,
			[152153] = 1986,
			[152178] = 2025,
			[151962] = 1992,
			[151994] = 1997,
			[152281] = 1983,
			[152058] = 1986,
			[152680] = 2031,
			[152122] = 2009,
			[152154] = 1997,
			[152686] = 2031,
			[151963] = 2004,
			[151995] = 1983,
			[152027] = 1987,
			[152059] = 1987,
			[152679] = 2031,
			[152123] = 1986,
			[152155] = 2004,
			[152025] = 1983,
			[151964] = 1983,
			[151996] = 2009,
			[152028] = 1992,
			[152060] = 1983,
			[152092] = 1983,
			[152124] = 2025,
			[152156] = 1984,
			[152283] = 2031,
			[151965] = 1985,
			[151997] = 1983,
			[152029] = 1986,
			[152061] = 2025,
			[152093] = 1984,
			[152125] = 1997,
			[152157] = 2009,
			[152423] = 2031,
			[151966] = 1983,
			[151998] = 1992,
			[152177] = 1986,
			[152062] = 1984,
			[152684] = 1984,
			[152126] = 2004,
			[152158] = 2025,
			[152282] = 1984,
			[151967] = 2009,
			[151999] = 2009,
			[152031] = 1992,
			[152063] = 1985,
			[152683] = 1984,
			[152127] = 1984,
			[152159] = 1986,
			[152416] = 2009,
			[151968] = 1987,
			[152000] = 1987,
			[152032] = 1997,
			[152064] = 2004,
			[152171] = 1986,
			[152128] = 2009,
			[152160] = 2025,
			[151937] = 1992,
			[151969] = 1997,
			[152001] = 1985,
			[152033] = 1984,
			[152169] = 1984,
			[152026] = 1984,
			[152129] = 1986,
			[152161] = 1997,
			[151938] = 2009,
			[151970] = 2025,
			[152002] = 1992,
			[152034] = 2004,
			[152038] = 1984,
			[152289] = 1986,
			[152130] = 2025,
			[152162] = 2004,
			[151939] = 2009,
			[151971] = 1986,
			[152003] = 1986,
			[152035] = 1985,
			[152284] = 1986,
			[152414] = 1986,
			[152131] = 1997,
			[152163] = 1984,
			[151940] = 1984,
			[151972] = 2031,
			[152004] = 1987,
			[152291] = 1987,
			[152135] = 1986,
			[152040] = 1986,
			[152132] = 2004,
			[152164] = 2009,
			[151941] = 1985,
			[151973] = 1987,
			[152005] = 2031,
			[152037] = 1983,
			[152412] = 2004,
			[152051] = 2004,
			[152133] = 1984,
			[152165] = 1986,
			[152036] = 1992,
		},
		["instance"] = {
			[669] = {
				["name"] = "Цитадель Адского Пламени",
				["bosses"] = {
					[1392] = "Кормрок",
					[1396] = "Килрогг Мертвый Глаз",
					[1438] = "Архимонд",
					[1427] = "Сокретар Вечный",
					[1372] = "Кровожад",
					[1394] = "Деспотичная Велари",
					[1425] = "Железный разоритель",
					[1447] = "Ксул'горак",
					[1391] = "Повелитель Скверны Закуун",
					[1395] = "Маннорот",
					[1426] = "Штурм Цитадели Адского Пламени",
					[1433] = "Повелитель теней Искар",
					[1432] = "Верховный совет Адского Пламени",
				},
			},
			[457] = {
				["name"] = "Литейная клана Черной горы",
				["bosses"] = {
					[1122] = "Дармак Повелитель Зверей",
					[959] = "Чернорук",
					[1123] = "Ка'граз Пламенная",
					[1202] = "Рудожуй",
					[1161] = "Груул",
					[1154] = "Горнило",
					[1203] = "Железные леди",
					[1147] = "Оператор Тогар",
					[1155] = "Ганс'гар и Франзок",
					[1162] = "Кромог",
				},
			},
			[768] = {
				["name"] = "Изумрудный Кошмар",
				["bosses"] = {
					[1744] = "Элерет Дикая Лань",
					[1703] = "Низендра",
					[1667] = "Урсок",
					[1704] = "Драконы Кошмара",
					[1750] = "Кенарий",
					[1726] = "Ксавий",
					[1738] = "Ил'гинот, Сердце Порчи",
				},
			},
			[786] = {
				["name"] = "Цитадель Ночи",
				["bosses"] = {
					[1737] = "Гул'дан",
					[1761] = "Верховный ботаник Тел'арн",
					[1743] = "Великий магистр Элисанда",
					[1713] = "Крос",
					[1762] = "Тихондрий",
					[1706] = "Скорпирон",
					[1725] = "Хрономатическая аномалия",
					[1751] = "Заклинательница клинков Алуриэль",
					[1731] = "Триллиакс",
					[1732] = "Звездный авгур Этрей",
				},
			},
			[861] = {
				["name"] = "Испытание доблести",
				["bosses"] = {
					[1830] = "Гарм",
					[1819] = "Один",
					[1829] = "Хелия",
				},
			},
			[875] = {
				["name"] = "Гробница Саргераса",
				["bosses"] = {
					[1861] = "Госпожа Сашж'ин",
					[1873] = "Аватара Падшего",
					[1862] = "Горот",
					[1896] = "Сонм страданий",
					[1897] = "Бдительная дева",
					[1856] = "Харджатан",
					[1867] = "Демоническая инквизиция",
					[1898] = "Кил'джеден",
					[1903] = "Сестры Луны",
				},
			},
			[946] = {
				["name"] = "Анторус, Пылающий Трон",
				["bosses"] = {
					[1992] = "Разрушитель миров Кин'гарота",
					[1985] = "Хранительница порталов Азабель",
					[2004] = "Кин'гарот",
					[1997] = "Военный совет Анторуса",
					[1986] = "Ковен шиварр",
					[2009] = "Ловец душ Имонар",
					[1983] = "Вариматрас",
					[1987] = "Гончие Саргераса",
					[2025] = "Эонар, Хранительница жизни",
					[1984] = "Агграмар",
					[2031] = "Аргус Порабощенный",
				},
			},
		},
	}
	-- LS 1 1 243290 GameObject-0-3151-1448-869-243290-00003B3C18 false
	local npcGUIDs = {
		-- HFC 
		[91331] = 1438, -- Архимонд
		[90199] = 1372, -- Кровожад
		[243290] = 1426, -- Ворота
		[92142] = 1432, -- Совет
		[92144] = 1432, -- Совет
		[92146] = 1432, -- Совет		
		[90284] = 1425, -- Робот
		[90316] = 1433, -- Искар
		[90378] = 1396, -- Килрогг
		[90435] = 1392, -- Кормрок		
		[91349] = 1395, -- Маннарот				
		[243567] = 1427, -- Сокретар ??
		[243566] = 1427, -- Сокретар ??	
		[90269] = 1394, -- Велари
		[93068] = 1447, -- Ксул
		[89890] = 1391, -- Закун
		
		
		--BRF
		[76865] = 1122,	--Дармак Повелитель Зверей
		[77325] = 959,  --Чернорук
		[76814] = 1123, --Ка'граз Пламенная
		[77182] = 1202, --Рудожуй
		[76877] = 1161, --Груул
		[76806] = 1154, --Горнило		
		[77477] = 1203, --Железные леди
		[77557] = 1203, --Железные леди
		[77231] = 1203,	--Железные леди
		[76906] = 1147, --Оператор Тогар
		[76973] = 1155, --Ганс'гар и Франзок
		[76974] = 1155, --Железные леди
		[77692] = 1162, --Кромог
		
		-- Emerald Nightmare
		[104636] = 1750, --Кенарий
		[254168] = 1750, --Сундук кенария --11:06:00 Loot GUIDs GameObject-0-3895-1520-5388-254168-00002C1354

		[105393] = 1738, -- Ил'гинот, Сердце Порчи
		
		[100497] = 1667, -- Урсок
		
		[102679] = 1704, -- Драконы Кошмара, Ysondre
		[102683] = 1704, -- Драконы Кошмара, Emeriss
		[102682] = 1704, -- Драконы Кошмара, Lethon
		[102681] = 1704, -- Драконы Кошмара, Taerar
			
		[102672] = 1703, -- Низендра
		[106087] = 1744, -- Элерет Дикая Лань
		
		[103769] = 1726, -- Ксавий
		
		-- The Nighthold
		
		[105503] = 1737, -- Гул'дан

		[104528] = 1761, -- Верховный ботаник Тел'арн
		[106643] = 1743, -- Великий магистр Элисанда
		[101002] = 1713, -- Крос
		[103685] = 1762, -- Тихондрий
		[102263] = 1706, -- Скорпирон

		[104415] = 1725, -- Хрономатическая аномалия
		[104731] = 1725, -- Хрономатическая аномалия
		[104676] = 1725, -- Хрономатическая аномалия
		[104491] = 1725, -- Хрономатическая аномалия
		[104492] = 1725, -- Хрономатическая аномалия
		
		[104881] = 1751, -- Заклинательница клинков Алуриэль
		[104327] = 1731, -- Триллиакс
		[104288] = 1731, -- Триллиакс лфп?
		[103758] = 1732, -- Звездный авгур Этрей

		-- Trial of Valor
		[114323] = 1830, -- Гарм
		[114537] = 1829, -- Хелия
		[114263] = 1819, -- Один	
		[260526] = 1819, -- Ящик одина
		--	19:46:55 Loot GUIDs GameObject-0-3773-1648-2690-260526-00002C8D3C
			
		[246036] = 1829, -- Ящик хелии?
		[266483] = 1737, -- Ящик гулдана
		[111022] = 1737, -- труп демона гулдан
		
		[248513] = 1713, -- Ящик кросса
		
		
		-- Анторус
		
		[122450] = 1992, -- Разрушитель миров Кин'гарота
		[122104] = 1985, -- Хранительница порталов Азабель		
		[122578] = 2004, -- Кин'гарот
		
		[122367] = 1997, -- Военный совет Анторуса
		[122369] = 1997, 
		[122333] = 1997,
		
		[122468] = 1986, -- Ковен шиварр
		[122467] = 1986, 
		[122469] = 1986, 
		[125436] = 1986,
		
		[124158] = 2009, -- Ловец душ Имонар
		[122366] = 1983, -- Вариматрас

		[126916] = 1987, -- Гончие Саргераса
		[126915] = 1987,
		
	--	[2025] = "Эонар, Хранительница жизни",
		[121975] = 1984, -- Агграмар
		[2031] = "Аргус Порабощенный",
	}
	
	-- for some reason it return encounterID ??
	local fixWrongBossId = {
		[1872] = 1743, -- Elisande
	}
	
	ns.fixWrongBossId = fixWrongBossId
	
	local BossIdToIndex = {} -- bossID - index
	local IndexToBossID = {} -- instanceID[index] - bossID
	
	function ns.GuidToID(guid)	
		if not guid then 
			return 0 
		else
			local id = guid:match("[^%-]+%-%d+%-%d+%-%d+%-%d+%-(%d+)%-%w+")
			return tonumber(id or 0)
		end
	end
	
	function ns.Cache_GetItemSourceByGUID(guid)	
		local id = ns.GuidToID(guid)	
		
		if npcGUIDs[id] then
			local bossID = npcGUIDs[id]
			local instanceID = LootHelperCache.encounters[bossID]

			return bossID, instanceID, LootHelperCache.instance[instanceID].bosses[bossID], LootHelperCache.instance[instanceID].name
		end
		
		return false
	end
	
	function ns.Cache_GetBossInfoByID(id)
		local bossID = npcGUIDs[id] or id
		local instanceID = LootHelperCache.encounters[bossID]
		
		if instanceID then
			return bossID, instanceID, LootHelperCache.instance[instanceID].bosses[bossID], LootHelperCache.instance[instanceID].name
		end
		
		return false
	end
	
	function ns.Cache_GetBossNameByID(id)
		local bossID = npcGUIDs[id] or id
		local instanceID = LootHelperCache.encounters[bossID]
		
		if instanceID then
			return LootHelperCache.instance[instanceID].bosses[bossID]
		end
		
		return false
	end
	
	function ns.Cache_GetInstanceByBossID(id)
		
		local instanceID = LootHelperCache.encounters[id]
		
		if instanceID then
			return instanceID, LootHelperCache.instance[instanceID].name
		end
		
		return false
	end
	
	local bossIDByNameFast = {}
	
	function ns.Cache_GetBossIDByName(raidID, name)		
	
		if raidID and name and bossIDByNameFast[raidID..name] then
			return bossIDByNameFast[raidID..name]
		end
		
		for bossID, bossName in pairs(LootHelperCache.instance[raidID].bosses) do		
			if bossName == name then
				bossIDByNameFast[raidID..name] = bossID
				return bossID
			end
		end		
	end
		
	function ns.Cache_GetNumBosses(instanceID)
		return #IndexToBossID[tonumber(instanceID)]
	end
	
	function ns.Cache_GetBossIDByIndex(instanceID, index)
		return IndexToBossID[tonumber(instanceID)][index]
	end
	
	local InstanceNameToID = {}
	
	function ns.Cache_GetInsanceIDByName(name)
		
		if InstanceNameToID[name] then
			return InstanceNameToID[name]
		end
		
		for id, value in pairs(LootHelperCache.instance) do
			
			if value.name == name then			
				InstanceNameToID[name] = id
				return id
			end
		end
		
		return false
	end

	local resetTimer
	local function ResetEvents()
		if(EncounterJournal) then
			EncounterJournal:RegisterEvent('EJ_LOOT_DATA_RECIEVED')
			EncounterJournal:RegisterEvent('EJ_DIFFICULTY_UPDATE')
		end

		resetTimer = nil
	end
	
	function ns.Cache_InitUpdateLocale()
	
		ns.Cache_UpdateLocaleNames(669)
		ns.Cache_UpdateLocaleNames(457)
		ns.Cache_UpdateLocaleNames(768)
		ns.Cache_UpdateLocaleNames(786)
		ns.Cache_UpdateLocaleNames(861)
		ns.Cache_UpdateLocaleNames(875)
		ns.Cache_UpdateLocaleNames(946)
		-- 768 EmeraldNightmare
		-- 786 Nighthold
		-- 861 Trial of Valor
		-- 875 Гробница
		-- 946 Анторус
	end
	
	function ns.Cache_UpdateLocaleNames(instanceID, diff)
		if(EncounterJournal) then
		   EncounterJournal:UnregisterEvent('EJ_LOOT_DATA_RECIEVED')
		   EncounterJournal:UnregisterEvent('EJ_DIFFICULTY_UPDATE')
		end

		EJ_SelectInstance(instanceID)
	
		if not diff then diff = 14 end--Default to "normal" in 6.0+ if diff arg not given.
		EJ_SetDifficulty(diff)--Make sure it's set to right difficulty or it'll ignore mobs (ie ra-den if it's not set to heroic). Use user specified one as primary, with curernt zone difficulty as fallback
	
		local instName = EJ_GetInstanceInfo(instanceID, true)

		local bossIndex = 1
		local bossName, description, bossID = EJ_GetEncounterInfoByIndex(bossIndex);

		while bossID do
			
			bossID = fixWrongBossId[bossID] or bossID
			
		--	print('T', instName, instanceID, bossIndex, bossName, bossID)
			ns.Cache_UpdateLocale(bossID, instanceID, bossName, instName)
			
			BossIdToIndex[bossID] = bossIndex
			
			IndexToBossID[instanceID] = IndexToBossID[instanceID] or {}
			IndexToBossID[instanceID][bossIndex] = bossID
			
			bossIndex = bossIndex + 1
			bossName, description, bossID = EJ_GetEncounterInfoByIndex(bossIndex);
		end
		
		if(resetTimer) then
			resetTimer:Cancel()
			resetTimer = nil
		end

		resetTimer = C_Timer.NewTimer(2, ResetEvents)
	end
	
	function ns.Cache_GetEncounterName(id)
		if id and LootHelperCache.instance[id] then
			return LootHelperCache.instance[id].name
		end
		
		return false
	end
	
	function ns.Cache_GetBossName(id)
		if id and LootHelperCache.encounters[id] then
			
			local encID = LootHelperCache.encounters[id]
			
			return LootHelperCache.instance[encID].bosses[id]
		end
	
		return false
	end
	
	function ns.Cache_GetItemIDInfo(itemID)
		
		if itemID and LootHelperCache.items[itemID] then
			local bossID = LootHelperCache.items[itemID]
			local instanceID = LootHelperCache.encounters[bossID]

			return bossID, instanceID, LootHelperCache.instance[instanceID].bosses[bossID], LootHelperCache.instance[instanceID].name
		end
		
		return false
	end
	
	function ns.Cache_AddItem(itemID, bossID, instanceID, bossName, instanceName)
		
		LootHelperDB = LootHelperDB or {}
		
		LootHelperDB.LootHelperCache = LootHelperDB.LootHelperCache or {}

		local LootHelperCache = LootHelperDB.LootHelperCache
		
		LootHelperCache.items = LootHelperCache.items or {}

		LootHelperCache.items[itemID] = bossID

		LootHelperCache.encounters = LootHelperCache.encounters or {}
		LootHelperCache.encounters[bossID] = instanceID

		LootHelperCache.instance = LootHelperCache.instance or {}

		LootHelperCache.instance[instanceID] = LootHelperCache.instance[instanceID] or {}
		LootHelperCache.instance[instanceID].name = instanceName
		LootHelperCache.instance[instanceID].bosses = LootHelperCache.instance[instanceID].bosses or {}
		LootHelperCache.instance[instanceID].bosses[bossID] = bossName
	end
	
	function ns.Cache_UpdateLocale(bossID, instanceID, bossName, instanceName)

		LootHelperCache.instance = LootHelperCache.instance or {}

		LootHelperCache.instance[instanceID] = LootHelperCache.instance[instanceID] or {}
		LootHelperCache.instance[instanceID].name = instanceName
		LootHelperCache.instance[instanceID].bosses = LootHelperCache.instance[instanceID].bosses or {}
		
		if not LootHelperCache.instance[instanceID].bosses[bossID] then
			print('LootHelper Error on BossID for', bossName, bossID)
		end
		
		LootHelperCache.instance[instanceID].bosses[bossID] = bossName
	end
	
	function ns.Cache_StartCaching(instanceID)
		
		
		if(EncounterJournal) then
		   EncounterJournal:UnregisterEvent('EJ_LOOT_DATA_RECIEVED')
		   EncounterJournal:UnregisterEvent('EJ_DIFFICULTY_UPDATE')
		end

		EJ_SetDifficulty(16)
		EJ_SelectInstance(instanceID)
		EJ_SetLootFilter(0, 0)

		local instName = EJ_GetInstanceInfo(instanceID, true)

		local bossIndex = 1
		local bossName, description, bossID = EJ_GetEncounterInfoByIndex(bossIndex);

		while bossID do

			EJ_SelectEncounter(bossID)


			for index = 1, EJ_GetNumLoot() do
			--	Wod Type of returns
			--	local name, texture, slot, itemClass, itemID, itemLink, encounterID = EJ_GetLootInfoByIndex(index)
				local itemID, encounterID, name, texture, slot, itemClass, itemLink = EJ_GetLootInfoByIndex(index);
					
				if name then
					print(name, itemID, bossID)
					
					ns.Cache_AddItem(itemID, bossID, instanceID, bossName, instName)
				end
			end

			bossIndex = bossIndex + 1

			bossName, description, bossID = EJ_GetEncounterInfoByIndex(bossIndex);

		end
		
		if(resetTimer) then
			resetTimer:Cancel()
			resetTimer = nil
		end

		resetTimer = C_Timer.NewTimer(2, ResetEvents)
	
	end
	
	function ns.Cache_StartCaching_Reset()
		LootHelperDB.LootHelperCache = {}
	end
end

if ns.debugUI then do
	local C = AleaUI_GUI
	C.LootHelper_debugFrames = {}
	
	local lastText = ''
	
	local listText = {}
	
	local function ShowMessage()
		
		local to = #listText
		local from = to - 50
		
		if from <= 0 then
			from = 1
		end
		
		lastText = ''
		
		for i=1, to do	
			lastText = lastText..listText[i]
		end
		
		C.LootHelper_debugFrames[1].main.infotext:SetText(lastText)
	end
	
	local function Update(self, panel, opts)
		
		self.free = false
		self:SetParent(panel)
		self:Show()	
		
		ShowMessage()
	end

	local function Remove(self)
		self.free = true
		self:Hide()
	end
	
	function ns.Debug_AddMessage(...)
		
		local num = select('#', ...)
		
		local msg = date("%H:%M:%S")
		
		for i=1, num do
			
			local arg = select(i, ...)
			
			if arg and type(arg) == 'string' then
				arg = arg:gsub('\n', '#n')
			end
			
			arg = tostring(arg)
			
			msg = msg..', '..arg
		end
		
		if num > 0 then
		
			table.insert(listText, msg..'\n')
		
			if C.LootHelper_debugFrames[1] and C.LootHelper_debugFrames[1].main.infotext:IsVisible() then
				ShowMessage()
			end
		end
	end
	
	local function CreateCoreButton(parent)
		local f = CreateFrame("Frame", nil, parent)
		f:SetSize(510, 310)
		f:SetFrameLevel(parent:GetFrameLevel() + 1)
		f.Scroll = CreateFrame("ScrollFrame", addOn.."DebugFramesScrollFrame", f, "UIPanelScrollFrameTemplate")
		f.Scroll:SetFrameLevel(f:GetFrameLevel() - 1)
		
		f.infotext = CreateFrame("EditBox", nil, f) 
		f.infotext:SetFontObject(ChatFontWhite)
		f.infotext:SetFont(ns.default_font, 12)
		f.infotext:SetFrameLevel(parent:GetFrameLevel())
		f.infotext:SetAutoFocus(false)
		f.infotext:SetWidth(800)
		f.infotext:SetHeight(300)	
		f.infotext:SetMultiLine(true)
		f.infotext:SetScript("OnEscapePressed", function(self)
			self:ClearFocus()
		end)
		f.infotext:SetScript('OnTextChanged', function(self, userInput)
			if userInput then				
				self:SetText(lastText)
			end
		end)
		
		f.Scroll:SetScrollChild(f.infotext)
		f.Scroll:SetPoint("TOPRIGHT", f, "TOPRIGHT", 0, -2)
		f.Scroll:SetPoint("TOPLEFT", f, "TOPLEFT", 0, -2)
		f.Scroll:SetPoint("BOTTOM", f, "BOTTOM", 0, 2)

		
		f.infotext:SetPoint('TOPLEFT', f.Scroll, "TOPLEFT", 11, 0)
		f.infotext:SetPoint('TOPRIGHT', f.Scroll, "TOPRIGHT", 0, 0)
		f.infotext:SetPoint("BOTTOM", f, "BOTTOM", 0, 2)
		f.infotext:SetHyperlinksEnabled(true)
		f.infotext:SetScript('OnHyperlinkClick', function(self, link, text, button)
			SetItemRef(link, text, button, self);
		end)
		
		
		f.Scroll:SetSize(505, 305)
		f.Scroll:SetHorizontalScroll(-5)
		f.Scroll:SetVerticalScroll(0)
		f.Scroll:EnableMouse(true)
		
		f.infotext:Show()
		f.Scroll:Show()
		f:Show()

		return f
	end

	function C:CreateLootHelper_DebugFrame()
		
		for i=1, #C.LootHelper_debugFrames do
			if C.LootHelper_debugFrames[i] then
				return C.LootHelper_debugFrames[i]
			end
		end
		
		local f = CreateFrame("Frame", nil, UIParent)
		f:SetSize(520, 320)
		f.free = true
		
		f.main = CreateCoreButton(f)
		f.main:ClearAllPoints()
		f.main:SetPoint("TOPLEFT", f, "TOPLEFT", 5, -5)
		f.main:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -25, 0)
		--[[
		f.bg = f:CreateTexture()
		f.bg:SetAllPoints()
		f.bg:SetTexture(1, 1, 0, 0.4)
		]]
		f.Update = Update
		f.Remove = Remove

		C.LootHelper_debugFrames[#C.LootHelper_debugFrames+1] = f
		
		return f
	end

	C.prototypes["LootHelper_DebugFrame"] = "CreateLootHelper_DebugFrame"
end end
--[==[
do
	
	local addon = ...
	local C = {}
	local addonChannel = "LHVCH"
	local remindMeagain_Guild = true
	local remindMeagain_Raid = true

	local name
	local string_match = string.match
	local format = format
	local SendAddonMessage = SendAddonMessage
	local tonumber = tonumber
	local sendmessagethottle = 20
	local versioncheck = 0
	
	-- http://bit.ly/loothelperupdate
	local downloadLink = '|cff60d3e1|Hloothelper|h[Скачать]|h|r'
	
	local SetHyperlink = ItemRefTooltip.SetHyperlink
	function ItemRefTooltip:SetHyperlink(data, ...)

		if data == 'loothelper' then
			ns:Popup(ns.downloadLink)
			return
		end
		
		SetHyperlink(self, data, ...)
	end
	
	
	if not IsAddonMessagePrefixRegistered(addonChannel) then
		RegisterAddonMessagePrefix(addonChannel)
	end

	function C:AddonMessage(msg, channel)

		if channel == "GUILD" and IsInGuild() then
			SendAddonMessage(addonChannel, msg, "GUILD")
		else
			local chatType = "PRINT"
			if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE) then
				chatType = "INSTANCE_CHAT"
			elseif IsInRaid(LE_PARTY_CATEGORY_HOME) then
				chatType = "RAID"
			elseif IsInGroup(LE_PARTY_CATEGORY_HOME) then
				chatType = "PARTY"
			end
				
			if chatType == "PRINT" then
				
			else
				SendAddonMessage(addonChannel, msg, chatType)
			end
		end
	end

	local function constructVersion(ver)
		local d1, d2, d3 = strsplit(".", ver)
		
		d1 = d1 or "0"
		d2 = d2 or "0"
		d3 = d3 or "0"
		
		if #d2 == 1 then
		   d2 = "00"..d2
		end
		if #d2 == 2 then
		   d2 = "0"..d2
		end
		if #d3 == 1 then
		   d3 = "00"..d3
		end
		if #d3 == 2 then
		   d3 = "0"..d3
		end
		
		return tonumber(d1..d2..d3)
	end

	local events = CreateFrame("Frame")
	events:SetScript("OnEvent", function(self, event, ...)
		self[event](self, event, ...)
	end)

	function events:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
		if prefix ~= addonChannel then return end
		if sender == name then return end
		
		if channel == 'GUILD' then
			if not remindMeagain_Guild then 
				return 
			end
		else
			if not remindMeagain_Raid then 
				return
			end
		end
		
		local version, source = strsplit(":", message)
		
		if version and source then
			local cntrV = constructVersion(version)
			local cntrmV = constructVersion(C.myVersionT)
		
			if cntrV > cntrmV then
				
				if channel == 'GUILD' then			
					remindMeagain_Guild = false
				else
					remindMeagain_Raid = false
				end
				
				print("|cFFFFFF00LootHelper: Доступна новая версия "..version..".|r "..downloadLink)
			end
		end
	end

	function events:SendAddonIndo()
		if GetTime() < versioncheck then return end
		versioncheck = GetTime() + sendmessagethottle
		C:AddonMessage(format("%s:%s", C.myVersionT, C.VersionSource))
	end

	function events:SendAddonIndo2()
		if GetTime() < versioncheck then return end
		versioncheck = GetTime() + sendmessagethottle
		C:AddonMessage(format("%s:%s", C.myVersionT, C.VersionSource) , "GUILD")
	end

	events.GROUP_ROSTER_UPDATE = events.SendAddonIndo
	events.PLAYER_ENTERING_WORLD = events.SendAddonIndo2
	events.PLAYER_ENTERING_BATTLEGROUND = events.SendAddonIndo
	events.GROUP_JOINED = events.SendAddonIndo
	events.RAID_INSTANCE_WELCOME = events.SendAddonIndo
	events.ZONE_CHANGED_NEW_AREA = events.SendAddonIndo

	events.GUILD_MOTD = events.SendAddonIndo2
	events.GUILD_NEWS_UPDATE = events.SendAddonIndo2
	events.GUILD_ROSTER_UPDATE = events.SendAddonIndo2

	events:RegisterEvent("PLAYER_LOGIN")

	function events:PLAYER_LOGIN()
		local version = ns.AddOnVer or GetAddOnMetadata(addon, "Version") or "0"
		local version_c = version:gsub("%.", "")
		
		name = UnitName("player").."-"..GetRealmName():gsub(' ', '')
		
		C.myVersionT = version
		C.myVersion = tonumber(version_c) or 0
		C.VersionSource = 'main'

		events:RegisterEvent("CHAT_MSG_ADDON")
		events:RegisterEvent("GROUP_ROSTER_UPDATE")
		events:RegisterEvent("PLAYER_ENTERING_WORLD")
		events:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND")
		events:RegisterEvent("GROUP_JOINED")
		events:RegisterEvent("RAID_INSTANCE_WELCOME")
		events:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		events:RegisterEvent("GUILD_MOTD")
		events:RegisterEvent("GUILD_NEWS_UPDATE")
		events:RegisterEvent("GUILD_ROSTER_UPDATE")
		
		events:SendAddonIndo()
		events:SendAddonIndo2()
	end

end
]==]
do
	--[[ Start popup creation ]]--
	
	local width = ns.downloadLink:len()*10
	
	local frame = CreateFrame("Frame", nil, UIParent)

	frame:SetBackdrop({
			bgFile = [[Interface\Buttons\WHITE8x8]],
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 1,
			insets = {left = 0, right = 0, top = 0, bottom =0}
		})
	frame:SetBackdropColor(0 , 0 , 0 , 0.7) --цвет фона
	frame:SetBackdropBorderColor(0 , 0 , 0 , 1) --цвет фона
	
	frame:SetSize(width, 40)
	frame:SetPoint("CENTER", UIParent, "CENTER")
	frame:SetFrameStrata("DIALOG")
	frame:Hide()

	local editBox = CreateFrame("EditBox", nil, frame)
	editBox:SetFontObject(ChatFontNormal)
	editBox:SetSize(width-40, 40)
	editBox:SetPoint("LEFT", frame, "LEFT", 10, 0)
	local hide = function(f) f:GetParent():Hide() end
	editBox:SetScript("OnEscapePressed", hide)
	editBox:SetScript('OnTextChanged', function(self, userInput)
		if userInput and self.lastText then				
			self:SetText(self.lastText)
		end
	end)
	
	local fsting = editBox:CreateFontString()
	fsting:SetFontObject(ChatFontNormal)
	fsting:SetFont(STANDARD_TEXT_FONT, 12, 'NONE')
	fsting:SetPoint("RIGHT", editBox, "RIGHT", -10, 0)
	fsting:SetText('|cFF606060Ctrl+C - '..L['COPY']..'\nCtrl+V - '..L['PASTE']..'|r')
	fsting:SetJustifyH('LEFT')
	
	local close_ = CreateFrame("Button", nil, frame)
	close_:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	close_:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	close_:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
	close_:SetSize(32, 32)
	close_:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
	close_:SetScript("OnClick", hide)
	--[[ End popup creation ]]--

	-- Avoiding StaticPopup taints by making our own popup, rather that adding to the StaticPopup list
	function ns:Popup(text)
		editBox.lastText = text
		editBox:SetText(text)
		editBox:HighlightText(0)
		editBox:GetParent():Show()
	end
end

do
	local lineHeight = 14
	local headerHeight = 0
	local lineGap = 3
	local disabledHeight = 2
	
	local menu = CreateFrame('Frame', nil)
	menu:EnableMouse(true)
	menu:SetSize(100, 20)
	menu:SetClampedToScreen(true)
	menu.buttons = {}
	menu:SetFrameStrata('TOOLTIP')
	menu:SetFrameLevel(20)
	menu.background = menu:CreateTexture(nil, 'OVERLAY')
	menu.background:SetAllPoints()
	menu.background:SetTexture(0,0,0,0.8)
	
	local function MenuButton_OnEnter(self)	
		if self.disabled then return end
		self.highlight:Show()
	end
	
	local function MenuButton_OnLeave(self)	
		self.highlight:Hide()
	end
	
	local function MenuButton_OnClick(self)	
		if self.disabled then return end
		
		if self.func then
			self.func(self)
		end
		
		menu:Hide()
	end
	
	local function MenuButton_OnMouseUp(self)
	
		self.text:SetPoint('TOPLEFT', self, 'TOPLEFT', 0, 0)
		self.text:SetPoint('BOTTOMRIGHT', self, 'BOTTOMRIGHT', 0, 0)
	end
	
	local function MenuButton_OnMouseDown(self)
		if self.disabled then return end
		self.text:SetPoint('TOPLEFT', self, 'TOPLEFT', 1, -1)
		self.text:SetPoint('BOTTOMRIGHT', self, 'BOTTOMRIGHT', 1, -1)
	end
	
	local function MenuButton_Cancel()
		menu:Hide()
	end
	
	local function CreateMenuButtons(i)
		local f = CreateFrame('Button', nil, menu)
		f:SetSize(100, lineHeight)
		f:SetFrameStrata('TOOLTIP')
		f:SetFrameLevel(menu:GetFrameLevel()+1)
		
		f.text = f:CreateFontString(nil, 'OVERLAY')
		f.text:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
		f.text:SetPoint('BOTTOMRIGHT', f, 'BOTTOMRIGHT', 0, 0)
		f.text:SetFontObject('GameFontWhite')
		
		f.highlight = f:CreateTexture(nil, "ARTWORK")
		f.highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight") --"Interface\\Buttons\\UI-Common-MouseHilight")
		f.highlight:SetBlendMode("ADD")
		f.highlight:Hide()
		f.highlight:SetAlpha(0.7)

		f.highlight:SetPoint("LEFT",   f, "LEFT",   0, 1)
		f.highlight:SetPoint("RIGHT",  f, "RIGHT",  0, -1)
		f.highlight:SetPoint("TOP",    f, "TOP",    0, 0)
		f.highlight:SetPoint("BOTTOM", f, "BOTTOM", 0, 0)
		
		f:SetScript('OnEnter', MenuButton_OnEnter)
		f:SetScript('OnLeave', MenuButton_OnLeave)
		f:SetScript('OnClick', MenuButton_OnClick)
		f:SetScript("OnMouseUp", MenuButton_OnMouseUp)
		f:SetScript("OnMouseDown", MenuButton_OnMouseDown)
		
		if menu.buttons[i-1] then
			f:SetPoint('TOP', menu.buttons[i-1], 'BOTTOM', 0, -lineGap)
		else
			f:SetPoint('TOP', menu, 'TOP', 0, -headerHeight-lineHeight)
		end
		
		menu.buttons[i] = f
		
		return f
	end
	
	function ns.HideMenu()	
		menu:Hide()
	end
	
	
	local menuFrame = CreateFrame("Frame", "LootHelperDatatextClickMenu", UIParent, "UIDropDownMenuTemplate")
	
	
	function ns.ShowMenu(parent, menuList)
		
		if true then
				
			EasyMenu(menuList, menuFrame, "cursor", -15, -7, "MENU", 2)
			return
		end
		
		
		if #menuList == 0 then 
			return 
		end
		
		menu:SetParent(parent:GetParent())
		menu:ClearAllPoints()
		menu:SetPoint('TOP', parent, 'BOTTOM', 0, 0)
		
		for i=1, #menu.buttons do
			menu.buttons[i].disabled = nil
			menu.buttons[i]:Hide()
		end
		
		local numButtons = 0
		local height = headerHeight+lineHeight
		
		for i=1, #menuList do
			numButtons = numButtons + 1
			
			local button = menu.buttons[i] or CreateMenuButtons(i)
			
			button.text:SetText(menuList[i].name)
			button.onClick = menuList[i].onClick
			button.disabled = menuList[i].disabled
			button:SetHeight(lineHeight)
			
			if button.disabled and button.isTitle ~= true then
				button:SetHeight(disabledHeight)
			end
			
			height = height + button:GetHeight() + lineGap

			button:Show()
			
			menu.buttons[i] = button
		end
		
		height = height + headerHeight
		
		menu:SetHeight(height)
		
		menu:Show()
		
		return menu
	end

end

do
	
	local C = AleaUI_GUI
	C.LootHelper_lockOutFrames = {}
	
	-- ns:GetUnitFullName(unit)
	local eventHandler = CreateFrame('Frame')
	eventHandler:SetScript('OnUpdate', function(self, elapsed)
		self.elapsed = ( self.elapsed or 0 ) + elapsed
		if self.elapsed < 1 then return end
		self.elapsed = 0
		
		if not C.LootHelper_lockOutFrames[1].main:IsVisible() then 
			self:Hide()
			return 
		end
		
		local raidID = ns.ForceShowRaidLockOut or ns:Client_GetEncounterLockoutID()
		
		local numMembers = IsInRaid() and GetNumGroupMembers() or 0
		
		local numBosses = 0
		
		if raidID then
			local raid = strsplit('_', raidID)
			numBosses = ns.Cache_GetNumBosses(raid) or 0
		end
		
		local width = 80/numBosses
		
		for i=1, numMembers do
			local name1, server = ns:GetUnitFullName('raid'..i)
			local tag = name1..'-'..server
			local name = ns.GetColoredName('raid'..i)
			
			local numBossesPlayer = ns:Client_GetEncounterLockoutBossNumber(raidID, tag)
			local lockout = ns:Client_GetEncounterLockoutInfoFast(raidID, tag)

			
			if lockout == 0 then -- with no raid info
				C.LootHelper_lockOutFrames[1].main.frames[i].texture:SetColorTexture(0.4,0.4,0,1)
				C.LootHelper_lockOutFrames[1].main.frames[i].bg:SetColorTexture(0.2,0.2,0,1)
			elseif lockout == 1 then -- with no lockout
				C.LootHelper_lockOutFrames[1].main.frames[i].texture:SetColorTexture(0,0.4,0,1)
				C.LootHelper_lockOutFrames[1].main.frames[i].bg:SetColorTexture(0,0.2,0,1)
			elseif lockout == 2 then -- with cooldown
				C.LootHelper_lockOutFrames[1].main.frames[i].texture:SetColorTexture(0.4,0,0,1)
				C.LootHelper_lockOutFrames[1].main.frames[i].bg:SetColorTexture(0.2,0.0,0,1)
			elseif lockout == -1 then -- with no data
				C.LootHelper_lockOutFrames[1].main.frames[i].texture:SetColorTexture(0.4, 0.4, 0.4,1)
				C.LootHelper_lockOutFrames[1].main.frames[i].texture:SetColorTexture(0.2, 0.2, 0.2,1)
			end
		
			C.LootHelper_lockOutFrames[1].main.frames[i].texture:SetWidth( numBossesPlayer > 0 and width*(numBosses - numBossesPlayer) or 80)
			
			C.LootHelper_lockOutFrames[1].main.frames[i].tag = tag
			C.LootHelper_lockOutFrames[1].main.frames[i].raidID = raidID
			C.LootHelper_lockOutFrames[1].main.frames[i].name:SetText(name)
			C.LootHelper_lockOutFrames[1].main.frames[i]:Show()
		end
		
		for i=numMembers+1 , 40 do
			C.LootHelper_lockOutFrames[1].main.frames[i].tag = nil
			C.LootHelper_lockOutFrames[1].main.frames[i].raidID = nil
			C.LootHelper_lockOutFrames[1].main.frames[i]:Hide()
		end
	end)
	eventHandler:Hide()
	
	local function Update(self, panel, opts)
		
		self.free = false
		self:SetParent(panel)
		self:Show()	
		
		eventHandler:Show()
		eventHandler.elapsed = 2
	end
	
	local function UpdateSize(self, panel, opts, parent)	
		if opts.width == 'full' then
			if parent then
				self:SetWidth( parent:GetWidth() - 15)
				self.main:SetWidth( parent:GetWidth() - 15)
			else
				self:SetWidth( panel:GetWidth() - 25)
				self.main:SetWidth( panel:GetWidth() - 25)
			end
		else
			self:SetWidth(520)
			self.main:SetWidth(520)
		end
	end
	
	local function Remove(self)
		self.free = true
		self:Hide()
		
		eventHandler:Hide()
	end
	

	local function CreateCoreButton(parent)
		local f = CreateFrame("Frame", nil, parent)
		f:SetSize(510, 310)
		f:SetFrameLevel(parent:GetFrameLevel() + 1)
		
		f.frames = {}
		
		for i=1, 40 do
		
			local button = CreateFrame("Frame", nil, f)
			button:SetSize(80, 16)
			
			button.name = button:CreateFontString(nil, 'OVERLAY')
			button.name:SetFont(STANDARD_TEXT_FONT, 12, 'OUTLINE')
			button.name:SetText('Button'..i)
			button.name:SetPoint('CENTER', button, 'CENTER', 0, 0)
			
			button.texture = button:CreateTexture(nil, 'ARTWORK')
			button.texture:SetDrawLayer('ARTWORK', 1)
			button.texture:SetColorTexture(0,0.4,0,1)
			button.texture:SetPoint('LEFT', button, 'LEFT', 0, 0)
			button.texture:SetSize(80, 16)
			
			button.bg = button:CreateTexture(nil, 'ARTWORK')
			button.bg:SetDrawLayer('ARTWORK', 0)
			button.bg:SetColorTexture(0.1,0.1,0.1,1)
			button.bg:SetAllPoints()
			
			button:SetScript('OnEnter', function(self)
				if self.tag and self.raidID then
			--		print(self.tag, self.raidID, 'OnEnter')	
					GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
					GameTooltip:ClearLines()
					
					ns:Client_FillGameTooltip(self.raidID, self.tag, GameTooltip)
					GameTooltip:Show()
				end
			end)
			button:SetScript('OnLeave', function(self)
				GameTooltip:Hide()
			end)
			
			if i == 1 then
				button:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
			elseif i%4 == 1 then
				button:SetPoint('TOP', f.frames[i-4], 'BOTTOM', 0, -10)
			else
				button:SetPoint('LEFT', f.frames[i-1], 'RIGHT', 10, 0)
			end

			f.frames[i] = button
		end
		
		f:SetScript('OnSizeChanged', function(self, width, height)
		--	print('Changed size ', width, height)
		end)
		
		f:Show()

		return f
	end

	function C:CreateLootHelper_LockOutFrame()
		
		for i=1, #C.LootHelper_lockOutFrames do
			if C.LootHelper_lockOutFrames[i] then
				return C.LootHelper_lockOutFrames[i]
			end
		end
		
		local f = CreateFrame("Frame", nil, UIParent)
		f:SetSize(520, 320)
		f.free = true
		
		f.main = CreateCoreButton(f)
		f.main:ClearAllPoints()
		f.main:SetPoint("TOPLEFT", f, "TOPLEFT", 5, -5)
		f.main:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -25, 0)
		--[[
		f.bg = f:CreateTexture()
		f.bg:SetAllPoints()
		f.bg:SetTexture(1, 1, 0, 0.4)
		]]
		f.Update = Update
		f.Remove = Remove
		f.UpdateSize = UpdateSize
		
		C.LootHelper_lockOutFrames[#C.LootHelper_lockOutFrames+1] = f
		
		return f
	end

	C.prototypes["LootHelper_LockOutFrame"] = "CreateLootHelper_LockOutFrame"
end

ns.RelicTypes = {
	["Blood"] = RELIC_SLOT_TYPE_BLOOD,
	["Shadow"] = RELIC_SLOT_TYPE_SHADOW,
	["Iron"] = RELIC_SLOT_TYPE_IRON,
	["Frost"] = RELIC_SLOT_TYPE_FROST,
	["Fire"] = RELIC_SLOT_TYPE_FIRE,
	["Fel"] = RELIC_SLOT_TYPE_FEL,
	["Arcane"] = RELIC_SLOT_TYPE_ARCANE,
	["Life"] = RELIC_SLOT_TYPE_LIFE,
	["Wind"] = RELIC_SLOT_TYPE_WIND,
	["Holy"] = RELIC_SLOT_TYPE_HOLY,
}

ns.ArtifactsPerClass = {
	['DEMONHUNTER'] = { 127829, 128832 },
	['DEATHKNIGHT'] = { 128402, 128292, 128403 },
	['DRUID']		= { 128858, 128860, 128821, 128306},
	['HUNTER']		= { 128861, 128826, 128808 },
	['MAGE']		= { 127857, 128820, 128862 },
	['MONK']		= { 128938, 128937, 128940 },
	['PALADIN']		= { 128823, 128866, 120978 },
	['PRIEST']		= { 128868, 128825, 128827 },
	['ROGUE']		= { 128870, 128872, 128476 },
	['SHAMAN'] 		= { 128935, 128819, 128911 },
	['WARLOCK']		= { 128942, 128943, 128941 },
	['WARRIOR']		= { 128910, 128908, 128289 },
}

ns.RelicSlots = {
	[127829] = { "Fel","Shadow","Fel"}, -- Havoc DH
	[128832] = {"Iron","Arcane","Fel"}, -- Vengeance DH

	[128402] = {"Blood","Shadow","Iron"}, -- Blood DK
	[128292] = {"Frost","Shadow","Frost"}, -- Frost DK
	[128403] = {"Fire","Shadow","Blood"}, -- Unholy DK

	[128858] = {"Arcane","Life","Arcane"}, -- Balance Druid
	[128860] = {"Frost","Blood","Life"}, -- Feral Druid
	[128821] = {"Fire","Blood","Life"}, -- Guardian Druid
	[128306] = {"Life","Frost","Life"}, -- Restoration Druid

	[128861] = {"Wind","Arcane","Iron"}, -- Beast Mastery Hunter
	[128826] = {"Wind","Blood","Life"}, -- Marksmanship Hunter
	[128808] = {"Wind","Iron","Blood"}, -- Survival Hunter

	[127857] = {"Arcane","Frost","Arcane"}, -- Arcane Mage
	[128820] = {"Fire","Arcane","Fire"}, -- Fire Mage
	[128862] = {"Frost","Arcane","Frost"}, -- Frost Mage

	[128938] = {"Life","Wind","Iron"}, -- Brewmaster Monk
	[128937] = {"Frost","Life","Wind"}, -- Mistweaver Monk
	[128940] = {"Wind","Iron","Wind"}, -- Windwalker Monk

	[128823] = {"Holy","Life","Holy"}, -- Holy Paladin
	[128866] = {"Holy","Iron","Arcane"}, -- Protection Paladin
	[120978] = {"Holy","Fire","Holy"}, -- Retribution Paladin

	[128868] = {"Holy","Shadow","Holy"}, -- Discipline Priest
	[128825] = {"Holy","Life","Holy"}, -- Holy Priest
	[128827] = {"Shadow","Blood","Shadow"}, -- Shadow Priest

	[128870] = {"Shadow","Iron","Blood"}, -- Assassination Rogue
	[128872] = {"Blood","Iron","Wind"}, -- Outlaw Rogue
	[128476] = {"Fel","Shadow","Fel"}, -- Subtlety Rogue

	[128935] = {"Wind","Frost","Wind"}, -- Elemental Shaman
	[128819] = {"Fire","Iron","Wind"}, -- Enhancement Shaman
	[128911] = {"Life","Frost","Life"}, -- Restoration Shaman

	[128942] = {"Shadow","Blood","Shadow"}, -- Affliction Warlock
	[128943] = {"Shadow","Fire","Fel"}, -- Demonology Warlock
	[128941] = {"Fel","Fire","Fel"}, -- Destruction Warlock

	[128910] = {"Iron","Blood","Shadow"}, -- Arms Warrior
	[128908] = {"Fire","Wind","Iron"}, -- Fury Warrior
	[128289] = {"Iron","Blood","Fire"}, -- Protection Warrior
}

ns.Artifacts = {
	[127829] = 577, -- Havoc DH
	[128832] = 581, -- Vengeance DH

	[128402] = 250, -- Blood DK
	[128292] = 251, -- Frost DK
	[128403] = 252, -- Unholy DK

	[128858] = 102, -- Balance Druid
	[128860] = 103, -- Feral Druid
	[128821] = 104, -- Guardian Druid
	[128306] = 105, -- Restoration Druid

	[128861] = 253, -- Beast Mastery Hunter
	[128826] = 254, -- Marksmanship Hunter
	[128808] = 255, -- Survival Hunter

	[127857] = 62, -- Arcane Mage
	[128820] = 63, -- Fire Mage
	[128862] = 64, -- Frost Mage

	[128938] = 268, -- Brewmaster Monk
	[128937] = 270, -- Mistweaver Monk
	[128940] = 269, -- Windwalker Monk

	[128823] = 65, -- Holy Paladin
	[128866] = 66, -- Protection Paladin
	[120978] = 70, -- Retribution Paladin

	[128868] = 256, -- Discipline Priest
	[128825] = 257, -- Holy Priest
	[128827] = 258, -- Shadow Priest

	[128870] = 259, -- Assassination Rogue
	[128872] = 260, -- Outlaw Rogue
	[128476] = 261, -- Subtlety Rogue

	[128935] = 262, -- Elemental Shaman
	[128819] = 263, -- Enhancement Shaman
	[128911] = 264, -- Restoration Shaman

	[128942] = 265, -- Affliction Warlock
	[128943] = 266, -- Demonology Warlock
	[128941] = 267, -- Destruction Warlock

	[128910] = 71, -- Arms Warrior
	[128908] = 72, -- Fury Warrior
	[128289] = 73, -- Protection Warrior	
}

do
	local artifactData = {
		equipped = {},		
		artifacts = {},
	}
	
	local initUpdate = true
	
	local function CTimer_AfterUpdate()
		initUpdate = true
		
		local _, class = UnitClass('player')

		if ns.ArtifactsPerClass[class] then
			
			for i = 16, 18, 1 do 
				if ns.Artifacts[GetInventoryItemID("player", i)] then
					local link = GetInventoryItemLink("player", i)
					
					local _, relicLink1 = GetItemGem(link, 1)
					local _, relicLink2 = GetItemGem(link, 2)
					local _, relicLink3 = GetItemGem(link, 3)
					
			--		print('Find artifact equpped:', link, 'R1:',relicLink1, 'R2:', relicLink2, 'R3:', relicLink3)		
					
					artifactData.equipped = {
						relicLink1,
						relicLink2,
						relicLink3
					}
				
					artifactData.artifacts[GetInventoryItemID("player", i)] = {
						equipped = true,
						link = link,
						relicLink1 = relicLink1,
						relicLink2 = relicLink2,
						relicLink3 = relicLink3,
						relicType1 = relicLink1 and select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink1))),
						relicType2 = relicLink2 and select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink2))),
						relicType3 = relicLink3 and select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink3))),
					}
				end
			end
			
			for i = 0, 4, 1 do -- (db.hidebag and -1) or
				for j = 1, GetContainerNumSlots(i), 1 do
					if ns.Artifacts[GetContainerItemID(i, j)] then
						local link = GetContainerItemLink(i, j)
						local _, relicLink1 = GetItemGem(link, 1)
						local _, relicLink2 = GetItemGem(link, 2)
						local _, relicLink3 = GetItemGem(link, 3)
						
						artifactData.artifacts[GetContainerItemID(i, j)] = {
							equipped = false,
							link = link,
							relicLink1 = relicLink1,
							relicLink2 = relicLink2,
							relicLink3 = relicLink3,
							relicType1 = relicLink1 and select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink1))),
							relicType2 = relicLink2 and select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink2))),
							relicType3 = relicLink3 and select(3, C_ArtifactUI.GetRelicInfoByItemID(ns.GetItemID(relicLink3))),
						}
				--		print('Find artifact in bags', link, 'R1:',relicLink1, 'R2:', relicLink2, 'R3:', relicLink3)
					end
				end
			end
		end
	end
	
	local artifactLoader = CreateFrame('Frame')	
		artifactLoader:RegisterEvent('PLAYER_LOGIN')
		artifactLoader:RegisterEvent('PLAYER_SPECIALIZATION_CHANGED')
		artifactLoader:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
		artifactLoader:RegisterEvent("ARTIFACT_CLOSE")
		
		artifactLoader:SetScript("OnEvent", function(self, event, unit)
		
			if event == 'PLAYER_SPECIALIZATION_CHANGED' then
				if unit == 'player' or not unit then
				
				else
					return
				end
			end
			
			if initUpdate then
				initUpdate = false
			--	print('T', event, unit)
				C_Timer.After(0.5, CTimer_AfterUpdate)
			end
		end)
		
	function ns.GetEquppedArtifactData()	
		return artifactData.equipped
	end
	
	function ns.GetArtifactsByType(relicType)
		local item1
		local item2
		local item3
		local item4
		
		for k,v in pairs(artifactData.artifacts) do	
			if v.equipped then
				if v.relicType1 == relicType or v.relicType2 == relicType or v.relicType3 == relicType then
					if not item1 then
						item1 = v.link
					elseif not item2 then
						item2 = v.link
					elseif not item3 then
						item3 = v.link
					elseif not item4 then
						item4 = v.link
					end
				end
			end
		end
		
		for k,v in pairs(artifactData.artifacts) do	
			if not v.equipped then
				if v.relicType1 == relicType or v.relicType2 == relicType or v.relicType3 == relicType then
					if not item1 then
						item1 = v.link
					elseif not item2 then
						item2 = v.link
					elseif not item3 then
						item3 = v.link
					elseif not item4 then
						item4 = v.link
					end
				end
			end
		end
		
		return item1, item2, item3, item4
	end
	
	
	ns.queueRelicPowerGain = {}
	
	
	local channelNameRelicNewInfo = 'LHRNI'
	if not IsAddonMessagePrefixRegistered(channelNameRelicNewInfo) then
		RegisterAddonMessagePrefix(channelNameRelicNewInfo)
	end
	ns.channelNameRelicNewInfo = channelNameRelicNewInfo
	
	
	-- ns:SendMessage(msg, tag)
	--  ns.RelicSync_GetNewItemInfo(cmsg, author, tag)
	
	function ns.RelicSync_GetNewItemInfo(info, author)	
	--	print('RelicSync_GetNewItemInfo', info, author)
		
		local relicData = { strsplit('\n', info) }
		
		local reason = relicData[1]
		local artifactID = relicData[2]
		local relicID = relicData[3]
		local powerID = relicData[4]
		
		ns.SpellIDByRelic[artifactID..'.'..relicID] = tonumber(powerID)
		
		if not LootHelperDB.missingRelics then
			LootHelperDB.missingRelics = {}
		end
		
		LootHelperDB.missingRelics[artifactID..'.'..relicID] = tonumber(powerID)
		
	--	print('Sync add', '['..artifactID..'.'..relicID..'] =', tonumber(powerID), 'by', author)
		
		if RelicInspector_SpellIDByRelic then
			RelicInspector_SpellIDByRelic[artifactID..'.'..relicID] = tonumber(powerID)
		end
	end
	
	function ns.GetNewRelicInfo(itemID,relicID, spellID)			
		if not ns.SpellIDByRelic[itemID..'.'..relicID] then
			ns.SpellIDByRelic[itemID..'.'..relicID] = spellID
			
			print('LootHelper: Find missing relic power', '['..itemID..'.'..relicID..'] =',spellID)
			
			
			ns.SendMessageGuild('New\n'..itemID..'\n'..relicID..'\n'..spellID, channelNameRelicNewInfo)	
			ns:SendMessage('New\n'..itemID..'\n'..relicID..'\n'..spellID, channelNameRelicNewInfo)
		elseif ns.SpellIDByRelic[itemID..'.'..relicID] and ns.SpellIDByRelic[itemID..'.'..relicID] ~= spellID then	
			local prev = ns.SpellIDByRelic[itemID..'.'..relicID]
			
			ns.SpellIDByRelic[itemID..'.'..relicID] = spellID
			
			print('LootHelper: Find missing relic power', '['..itemID..'.'..relicID..'] =',spellID)
			
			ns.SendMessageGuild('Correct\n'..itemID..'\n'..relicID..'\n'..spellID, channelNameRelicNewInfo)	
			ns:SendMessage('Correct\n'..itemID..'\n'..relicID..'\n'..spellID, channelNameRelicNewInfo)
		end
	end
	
	function ns.HaveInfoForThisRelic(artifactID, relicID, spellID)	
		return ns.SpellIDByRelic[artifactID..'.'..relicID]
	end
	
	local function IsValidRelicType(artifactID, relicType )
		if ns.RelicSlots[artifactID] then
			
			for i=1, #ns.RelicSlots[artifactID] do
				if ns.RelicSlots[artifactID][i] == relicType then
					return true
				end
			end
		end
		
		return false
	end
	
	local function RelicFinder(itemIDorLink, from)
		if IsArtifactRelicItem(itemIDorLink) then
			
			local itemID = type(itemIDorLink) == 'string' and ns.GetItemID(itemIDorLink) or itemIDorLink
			local relicName, relicIcon, relicType, relicLink
			
			local artifactID = C_ArtifactUI.GetArtifactInfo() or C_ArtifactUI.GetEquippedArtifactInfo();
	
			if itemID then
				relicName, relicIcon, relicType, relicLink = C_ArtifactUI.GetRelicInfoByItemID(itemID)
			end
		
			
			if relicName and IsValidRelicType(artifactID, relicType ) then
				
			--	print('RelicFinder: info by', itemIDorLink, 'Name:', GetItemInfo(itemID), 'Power:', ns.HaveInfoForThisRelic(artifactID, itemID) and GetSpellInfo(ns.HaveInfoForThisRelic(artifactID, itemID)) )				
			--	print('  Tag:', '['..itemID..'.'..artifactID ..'] =', ns.HaveInfoForThisRelic(artifactID, itemID))
				
				ns.queueRelicPowerGain[itemID] = true
				
				local powerID = C_ArtifactUI.GetPowersAffectedByRelicItemLink(relicLink)
				local powerLink = powerID and C_ArtifactUI.GetPowerLinks(powerID) or false
	
				if powerID then
					local powerInfo = C_ArtifactUI.GetPowerInfo(powerID);
					local name, _, _, _, _, _, spellID2 = GetSpellInfo(powerInfo.spellID)
						
					--	print('T', spellID2, powerInfo.spellID)
						
					ns.GetNewRelicInfo(artifactID, itemID, powerInfo.spellID)	
				else				
					if ns.artifactTraits[artifactID] then
						-- We have cached power ids
						
						local powerBonus = nil
						
						ns.parserTooltip:ClearLines()
						ns.parserTooltip:SetHyperlink(relicLink)	
		
						for i=1, ns.parserTooltip:NumLines() do        
							local left = _G[ns.parserTooltip:GetName().."TextLeft"..i]:GetText()
							
							if left then			
								for powerID, spellID in pairs(ns.artifactTraits[artifactID]) do
									if string.match(left, (GetSpellInfo(spellID))) then
										powerBonus = spellID
										break
									end
								end
							end
						end
						
						if powerBonus then
							ns.GetNewRelicInfo(artifactID, itemID, powerBonus)		
						end
					end
				end
			else
			--	print('RelicFinder: info by', itemIDorLink, 'Name:', GetItemInfo(itemID), 'Power:', ns.HaveInfoForThisRelic(artifactID, itemID) and GetSpellInfo(ns.HaveInfoForThisRelic(artifactID, itemID)) )				
			--	print('  Tag:', '['..itemID..'.'..artifactID ..'] =', ns.HaveInfoForThisRelic(artifactID, itemID))
			end
		else
		--	print('RelicFinder: Item is not a relic', itemIDorLink, (select(2, GetItemInfo(itemIDorLink))) )
		end
	end
	ns.RelicFinder = RelicFinder
	
	--[==[
	local relicLootHandler = CreateFrame('Frame')
	relicLootHandler:RegisterEvent('CHAT_MSG_LOOT')
	relicLootHandler:SetScript('OnEvent', function(self,event, msg,sender)		
		RelicFinder(msg)
	end)
	]==]
	--[==[
	local ignoreFuncCall = false
	hooksecurefunc('IsArtifactRelicItem', function(itemIDOrLink)
		if not ignoreFuncCall then
			ignoreFuncCall = true
			RelicFinder(itemIDOrLink, 'IsArtifactRelicItem')
		end
		ignoreFuncCall = false
	end)
	]==]
	
	
	local lastLink = nil
	GameTooltip:HookScript("OnTooltipSetItem", function(self)
		local _, link = self:GetItem()
		if not link or lastLink == link then 
			return; 
		end

		lastLink = link

		RelicFinder(link, 'GameTooltip')
	end)
	GameTooltip:HookScript("OnHide", function(self)
		lastLink = nil
	end)
	
end



-- Artifact parses
do
	-- Artifact Parser
	local function PrepareForScan()
		local ArtifactFrame = _G.ArtifactFrame
		
		if not ArtifactFrame or not ArtifactFrame:IsShown() then
	--		print('PrepareForScan', 'Success')
			
			_G.UIParent:UnregisterEvent("ARTIFACT_UPDATE")
			if ArtifactFrame then
				ArtifactFrame:UnregisterEvent("ARTIFACT_UPDATE")
			end
		end
	end

	local function RestoreStateAfterScan()
		local ArtifactFrame = _G.ArtifactFrame
			
		if not ArtifactFrame or not ArtifactFrame:IsShown() then
			C_ArtifactUI.Clear()
			
	--		print('RestoreStateAfterScan', 'Success')
			
			if ArtifactFrame then
				ArtifactFrame:RegisterEvent("ARTIFACT_UPDATE")
			end
			_G.UIParent:RegisterEvent("ARTIFACT_UPDATE")
		end
	end

	local lastUpdate = -1
	local init = true
	local initUpdate = true

		
	function ns:ResetArifactPercInfo()
		lastUpdate = -1
	end
	
	function ns:GetAtrifactPercInfo()
		initUpdate = true
		
		if not HasArtifactEquipped() then		
			lastUpdate = -1			
			return false
		end
		
		if init or lastUpdate < GetTime() then
			lastUpdate = GetTime() + 2
			init = false

			PrepareForScan()
			
			local opennedArtifact = C_ArtifactUI.GetArtifactInfo()
			local equippedArtifact = C_ArtifactUI.GetEquippedArtifactInfo()
			
			
			if not opennedArtifact then			
				SocketInventoryItem(INVSLOT_MAINHAND)
			end
			
			local powers = C_ArtifactUI.GetPowers();
			local itemID = C_ArtifactUI.GetArtifactInfo() or C_ArtifactUI.GetEquippedArtifactInfo();
			
		--	print('T', powers)
			
			if powers then
				
				if not ns.artifactTraits then ns.artifactTraits = {} end
	
				if itemID and 
					not ns.artifactTraits[itemID] then
						ns.artifactTraits[itemID] = {}
				end
				
				
				for i, powerID in ipairs(powers) do
					--[==[
						self.powerID = powerID;
						self.spellID = powerInfo.spellID;
						self.currentRank = powerInfo.currentRank;
						self.bonusRanks = powerInfo.bonusRanks;
						self.maxRank = powerInfo.maxRank;
						self.isStart = powerInfo.isStart;
						self.isGoldMedal = powerInfo.isGoldMedal;
						self.isFinal = powerInfo.isFinal;
						self.tier = powerInfo.tier;
						self.textureKit = textureKit;
						self.linearIndex = powerInfo.linearIndex;
						self.numMaxRankBonusFromTier = powerInfo.numMaxRankBonusFromTier	
					]==]
					
					local powerInfo = C_ArtifactUI.GetPowerInfo(powerID);
					
					local spellID, cost, currentRank, maxRank, bonusRanks
					
					if powerInfo and type(powerInfo) == 'table' then
						spellID = powerInfo.spellID;
						maxRank = powerInfo.maxRank;
					else
						spellID, cost, currentRank, maxRank, bonusRanks = C_ArtifactUI.GetPowerInfo(powerID);	
					end
				
					if maxRank and maxRank > 1 and maxRank < 7 then
						ns.artifactTraits[itemID][powerID] = spellID
					end
				end
			end
			
			for i=1, 3 do
				local _, _, _, itemLink = C_ArtifactUI.GetRelicInfo(i);
			
				ns.RelicFinder(itemLink)
			end
			
			for k,v in pairs(ns.queueRelicPowerGain) do				
				if v == true then
					ns.RelicFinder(k)
				end
			end
			
			RestoreStateAfterScan()
		end
	end
	
	local function CTimer_AfterUpdate()
		ns:GetAtrifactPercInfo()
	end
	
	local eventframe = CreateFrame("Frame")
	eventframe:SetScript("OnEvent", function(self, event, newItem)
	
	--	print(event, newItem)
		if event == 'ARTIFACT_UPDATE' then
			if newItem then
				if initUpdate then
					initUpdate = false
					C_Timer.After(0.5, CTimer_AfterUpdate)
				end
			end
		else
			if initUpdate then
				initUpdate = false
				C_Timer.After(0.5, CTimer_AfterUpdate)
			end
		end
	end)
	
	eventframe:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	eventframe:RegisterEvent("PLAYER_TALENT_UPDATE")
	eventframe:RegisterEvent("PLAYER_LEVEL_UP")
	eventframe:RegisterEvent("PLAYER_LOGIN")
	
	eventframe:RegisterEvent("ARTIFACT_UPDATE")
--	eventframe:RegisterEvent("ARTIFACT_CLOSE")
	
	eventframe:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
end

do
	-- WeakAuras code
	local coroutine = coroutine
	local dynFrame = {}
	ns.dynFrame = dynFrame
	
	dynFrame.frame = CreateFrame("frame");
	dynFrame.update = {};
	dynFrame.size = 0;

	function dynFrame.AddAction( name, func)
		if not name then
			name = fmt("NIL", dynFrame.size+1);
		end

		if not dynFrame.update[name] then
			dynFrame.update[name] = func;
			dynFrame.size = dynFrame.size + 1
			dynFrame.frame:Show();
		end
	end

	function dynFrame.RemoveAction(self, name)
		if dynFrame.update[name] then
			dynFrame.update[name] = nil;
			dynFrame.size = dynFrame.size - 1
			if dynFrame.size == 0 then
				dynFrame.frame:Hide();
			end
		end
	end

	dynFrame.frame:Hide();
	dynFrame.frame:SetScript("OnUpdate", function(self, elapsed)

		local start = debugprofilestop();
		local hasData = true;

		while (debugprofilestop() - start < 16 and hasData) do
			hasData = false;

			for name, func in pairs(dynFrame.update) do

				hasData = true;

				if coroutine.status(func) ~= "dead" then
					local err,ret1,ret2 = assert(coroutine.resume(func))
					if err then

					end
				else
					dynFrame:RemoveAction(name);
				end
			end
		end
	end);
end